/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef   __CORTOS_H
#define   __CORTOS_H

#include "cortos_cpu.h"
#include "cortos_conf.h"
#include "cortos_type.h"
#include "cortos_dai.h"

extern void    CORT_Init(void);
extern void    CORT_Start(void);
extern void    CORT_StateGo(void);
extern t_u16   CORT_CpuUsage(void);
extern void    CORT_SchedLock(void);
extern void    CORT_SchedUnLock(void);
extern t_u32   CORT_GetTickCounter(void);

extern t_s8    CORT_Task_Create(void (*task_entry)(void *p_arg), void *parg, t_stk *pstk, t_size stksize, t_u8 prio, t_u8 *pName, t_tskid *pThreadID);
extern t_s8    CORT_Task_Delete(t_tskid ThreadID);
extern t_s8    CORT_Task_Suspend(t_tskid ThreadID);
extern t_s8    CORT_Task_Resume(t_tskid ThreadID);
extern t_s8    CORT_Task_GetID(t_tskid *pThreadID);
extern void    CORT_Task_Delay(t_s32 tick);
extern t_s8    CORT_Task_GetState(t_tskid ThreadID, t_tasksta * pState);
extern t_tskid CORT_Task_GetNum(void);
extern t_u8    CORT_Task_GetNext(t_tskid CurID, t_tskid * pNextID);

extern t_s8    CORT_Sem_Create(t_sem * psem, t_u32 init_val);
extern t_s8    CORT_Sem_Delete(t_sem * psem);
extern t_s8    CORT_Sem_Obtain(t_sem * psem, t_s32 tick);
extern t_s8    CORT_Sem_Release(t_sem * psem);

extern t_s8    CORT_Queue_Create(t_queue *pqueue, t_void ** ppbuf, t_u32 num);
extern t_s8    CORT_Queue_Delete(t_queue *pqueue);
extern t_s8    CORT_Queue_Write(t_queue *pqueue, t_void * pbuf, t_bool bRear, t_s32 tick);
extern t_s8    CORT_Queue_Read(t_queue *pqueue, t_void ** ppbuf, t_bool bFront, t_s32 tick);

extern t_s8    CORT_Pipe_Create(t_pipe *ppipe, t_u8 *pbuf, t_u16 num);
extern t_s8    CORT_Pipe_Delete(t_pipe *ppipe);
extern t_s8    CORT_Pipe_Write(t_pipe *ppipe, t_u8 * pbuf, t_s32 *pnum, t_s32 tick);
extern t_s8    CORT_Pipe_Read(t_pipe *ppipe, t_u8 * pbuf, t_s32 *pnum, t_s32 tick);
extern t_s8    CORT_Pipe_Reset(t_pipe *ppipe);

extern t_s8    CORT_Mailbox_Create(t_mailbox * pmail);
extern t_s8    CORT_Mailbox_Delete(t_mailbox * pmail);
extern t_s8    CORT_Mailbox_Write(t_mailbox * pmail, void * pbuf, t_s32 tick);
extern t_s8    CORT_Mailbox_Read(t_mailbox * pmail, void ** ppbuf, t_s32 tick);

extern t_s8    CORT_Flag_Create(t_flag *pflag, t_u32 init_val);
extern t_s8    CORT_Flag_Delete(t_flag *pflag);
extern t_s8    CORT_Flag_Obtain(t_flag *pflag, t_u32 *pval, t_u32 mask, t_u8 oper, t_s32 tick);
extern t_s8    CORT_Flag_Set(t_flag *pflag, t_u32 mask, t_u8 oper);

extern t_s8    CORT_Timer_Create(t_timer *ptimer, t_u8 flag, t_u32 inittick, t_u32 schedtick, t_timproc func);
extern t_s8    CORT_Timer_Delete(t_timer *ptimer);
extern t_s8    CORT_Timer_Reset(t_timer *ptimer);
extern t_s8    CORT_Timer_Start(t_timer *ptimer, t_u32 inittick, t_s32 loopcnt);
extern t_s8    CORT_Timer_Stop(t_timer *ptimer);
extern t_s8    CORT_Timer_Set(t_timer *ptimer, t_u8 flag, t_u32 schedtick);

extern t_s8    CORT_Interrupt_Register(t_u16 idx,void (*task_entry)(t_u16));

extern void    CORT_Hook_RegCall(t_hook * pHook);

#endif

