/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "cortos.h"
#include "cortos_common.h"

/*******************************************************************************
**
**
*******************************************************************************/
t_bool         CORT_Running     = m_false;
t_u16          CORT_CPUUsageVal = 0;
t_u8           CORT_OSStatus    = 0;
t_u32          CORT_TickCnt     = 0;
t_task        *CORT_CurTcb 	    = m_null;
t_u8           CORT_CurPrio     = CORT_LOWEST_PRIO+1;
t_u8           CORT_HighPrio    = 0;
t_tskid        CORT_HeapNum     = 0;
t_task        *CORT_HeapFirst   = m_null;

t_tklist       CORT_FreeList    = {0};
t_tklist       CORT_AliveList   = {0};


static t_tskid CORT_IdleID		= 0;

t_u8           CORT_IntNesting  = 0;
/*******************************************************************************
**
**
*******************************************************************************/
#if CORT_KERNEL_LITE
t_prio         CORT_ReadyTable  = 0;
static t_task  CORT_TaskHeap [CORT_LOWEST_PRIO + 1];
#endif
#if CORT_KERNEL_FULL
static t_task  CORT_TaskHeap [CORT_MAX_TASK];
#endif

#if (CORT_TIMER_EN)
t_timer *      CORT_TimerList   = m_null;
#endif

#if (CORT_SCHEDLOCK_EN)
static t_u16   CORT_LockCnt     = 0;
#endif

#if (CORT_CALC_CPUUSAGE)
static t_bool  CORT_StatusRdy	= m_false;
static t_u32   CORT_StateCnt    = 0;
#if (CORT_TASK_RUNTIME_EN)
static t_u32   CORT_IdleUsec	= 0;
#else
static t_u32   CORT_IdleCntMax	= 0;
static t_u32   CORT_IdleCnt     = 0;
#endif
#endif

#if CORT_INTE_STK_SIZE
t_stk         *CORT_IntTsk_top  = m_null;
t_stk          CORT_IntTsk[CORT_INTE_STK_SIZE];
#endif

static t_stk   CORT_IdleTask_Stk[CORT_IDLE_STK_SIZE];
#if (CORT_TIMER_EN || CORT_CALC_CPUUSAGE)
static t_stk   CORT_HighTask_Stk[CORT_HIGH_STK_SIZE];
#endif

#if (CORT_HOOK_EN)
t_hook         CORT_Hook		= {0};
#endif
/*******************************************************************************
**
**
*******************************************************************************/
/*
**
*/
void CORT_CommInit(void)
{
	t_tskid i;

	CORT_MemSet((t_void*)CORT_TaskHeap, 0, sizeof(CORT_TaskHeap));

	CORT_HeapFirst = CORT_TaskHeap;
	CORT_HeapNum   = CORT_ARRAYSIZE(CORT_TaskHeap);

	for (i = 0; i <= CORT_HeapNum; i++)
	{
		CORT_TaskHeap[i].tk_stack_top = m_null;
		CORT_TaskHeap[i].tk_thread_id = i;
		CORT_TaskHeap[i].tk_list_task.ti_task = &CORT_TaskHeap[i];
		
		CORT_ListInsert(&CORT_FreeList, &CORT_TaskHeap[i].tk_list_task);
	}

#if (CORT_INTE_STK_SIZE)
#if (CORT_PORTING_STK_GROWTH)
	CORT_IntTsk_top = &CORT_IntTsk[CORT_INTE_STK_SIZE-1];
#else
	CORT_IntTsk_top =  CORT_IntTsk;
#endif
#endif
}

/*
**
*/
t_void CORT_MemSet(void *pMem, t_u8 val, t_size size)
{	
	t_u8 *pDat = (t_u8*)pMem;
    while (size > 0) {
        *pDat++ = val;
        size--;
    }
}

/*
**
*/
#if (CORT_FLAG_EN)
t_u8 CORT_CheckFlag(t_u32 *pval, t_u32 *flag, t_u32 mask, t_u8 oper)
{
	t_u8 operation;
	t_u32 flagrdy,val;
	t_bool consume;
	
	if (oper & CORT_CONSUME)
	{
		consume = m_true;
		operation = oper & (~CORT_CONSUME);
	}
	else
	{
		consume = m_false;
		operation = oper;
	}

	val = *flag;
	switch(operation)
	{
	case CORT_SET_ALL: 
		flagrdy = val  & mask;
		if (flagrdy == mask)
		{
			if (consume)
			{
				*flag = val & (~flagrdy);
			}
			if (pval)
			{
				*pval = flagrdy;
			}
		    return CORT_OK;
		}
		break;
	case CORT_SET_ANY:
		flagrdy = val & mask;
		if (flagrdy != 0)
		{
			if (consume)
			{
				*flag = val & (~flagrdy);
			}
			if (pval)
			{
				*pval = flagrdy;
			}
		    return CORT_OK;
		}
		break;
	case CORT_CLR_ALL: 
		flagrdy = ~val & mask;
		if (flagrdy == mask)
		{
			if (consume)
			{
				*flag = val | flagrdy;
			}
			if (pval)
			{
				*pval = flagrdy;
			}
		    return CORT_OK;
		}
		break;
	case CORT_CLR_ANY:
		flagrdy = ~val & mask;
		if (flagrdy != 0)
		{
			if (consume)
			{
				*flag = val | flagrdy;
			}
			if (pval)
			{
				*pval = flagrdy;
			}
		    return CORT_OK;
		}
		break;
	default:;
		return CORT_ERR_INVALID_PARAM;
	}
	return CORT_ERROR;
}
#endif

/*
**
*/
void CORT_ListInsert(t_tklist *pList, t_tkitem *pItem)
{
	if (pList->tl_tail)
	{
		pList->tl_tail->ti_next = pItem;

		pItem->ti_prev = pList->tl_tail;
		pItem->ti_next = m_null;
		pList->tl_tail = pItem;	
	}
	else
	{
		pItem->ti_prev = m_null;
		pItem->ti_next = m_null;
		pList->tl_head = pItem;
		pList->tl_tail = pItem;
	}
	pList->tl_cnt++;
}

/*
**
*/
void CORT_ListRemove(t_tklist *pList, t_tkitem *pItem)
{
	t_tkitem *ptemp;

	ptemp = pItem->ti_prev;
	if (ptemp == m_null)
	{
		pList->tl_head = pItem->ti_next;
	}
	else
	{
		ptemp->ti_next = pItem->ti_next;
	}
	ptemp = pItem->ti_next;
	if (ptemp == m_null)
	{
		pList->tl_tail = pItem->ti_prev;
	}
	else
	{
		ptemp->ti_prev = pItem->ti_prev;
	}
	pList->tl_cnt--;
}

/*
**
*/
t_void CORT_IdleTask(void *pd)
{
	while(1)
	{
		CORT_EnterCritical();
#if (CORT_HOOK_EN && CORT_HOOK_SLEEP_EN)
		if (CORT_Hook.hk_sleep)
		{
			CORT_Hook.hk_sleep();
		}
#endif
#if (CORT_CALC_CPUUSAGE && !CORT_TASK_RUNTIME_EN)
 		CORT_IdleCnt++;
#endif		
		CORT_ExitCritical();
	}
}

#if (CORT_TASK_CHECK_STK_EN)
/*
** Clear stack memory
*/
t_void CORT_ClearStk(t_stk * pstk, t_size size)
{
	CORT_MemSet(pstk, 0xCC, size);
}

/*
** Check stack state
*/
t_void CORT_CalcTaskStk(t_task * ptcb)
{
	t_size stkFree = 0;
	t_size stkSize = ptcb->tk_stack_size;
	t_stk* pStk, Val = (t_stk)0xCCCCCCCC;
	
 #if (CORT_PORTING_STK_GROWTH)
 	pStk = ptcb->tk_stack_bottom;
 	while((*pStk++)==Val && stkFree < stkSize)
	{
		stkFree += sizeof(t_stk);
	}
 #else
 	pStk = ptcb->StackBottom + stkSize - 1;
 	while((*pStk--)==Val && stkFree < stkSize)
	{
		stkFree++;
	}
 #endif

 	ptcb->tk_stack_free = stkFree;
}

/*
** Check stack state
*/
void CORT_CheckTaskStk(t_task * ptcb)
{
	t_stk* pStk, Val = (t_stk)0xCCCCCCCC;
		
	if (ptcb)
	{		
#if (CORT_PORTING_STK_GROWTH)
		pStk = ptcb->tk_stack_bottom;
#else
		pStk = ptcb->StackBottom + ptcb->tk_stack_size - 1;
#endif
		if ((*pStk) != Val)
		{
			/* Stack Overflow */
			CORT_TaskError(ptcb);
			
#if (CORT_HOOK_EN && CORT_HOOK_TASK_ERROR_EN)
			if (CORT_Hook.hk_task_error)
			{
				CORT_Hook.hk_task_error(ptcb->tk_thread_id,
					CORT_ERR_STACKOVERFLOW);
			}
#endif
		}
	}
}
#endif

/*
**
*/
#if (CORT_TIMER_EN)
t_void CORT_TimerProc(void)
{
	t_s32 cnt;
	t_timer *ptimer = CORT_TimerList;

	CORT_EnterCritical();
	CORT_OSStatus |= CORT_OS_STATUS_TIMING;
	CORT_ExitCritical();
	while(ptimer)
	{
		if (
#if (CORT_CHECK_ALIVE)
			(ptimer->tm_alive_flag == CORT_ALIVE_FLAG_TIMER) &&
#endif
			(ptimer->tm_remain && (--ptimer->tm_remain) == 0))
		{
			cnt = ptimer->tm_loopcnt;

			if (cnt != 1)
			{
				ptimer->tm_remain = ptimer->tm_reload;
			}
			if (cnt >= 1)
			{
				ptimer->tm_loopcnt = --cnt;
			}
			ptimer->tm_func(ptimer->tm_flag, (!ptimer->tm_remain) ? m_true : m_false);
		}
		ptimer = ptimer->tm_next;
	}
	CORT_EnterCritical();
	CORT_OSStatus &= ~CORT_OS_STATUS_TIMING;
	CORT_ExitCritical();
}
#endif

/*
**
*/
#if (CORT_CALC_CPUUSAGE)
t_void CORT_StateProc(void)
{
	t_u32 run;
	t_s16 usage;
#if CORT_TASK_RUNTIME_EN
	t_task * ptcb;
#endif
	
	if (CORT_StatusRdy && (++CORT_StateCnt)>= (CORT_TICK_PER_SEC / CORT_STAT_PRE_SEC))
	{
		CORT_StateCnt = 0;
#if CORT_TASK_RUNTIME_EN
		ptcb = &CORT_TaskHeap[CORT_IdleID];
		CORT_EnterCritical();
		run  = ptcb->tk_rtim_usec;
	    CORT_ExitCritical();

		#define USCNT_MAX	(CORT_TICK_PER_SEC*1000 / CORT_STAT_PRE_SEC)
		usage = (t_s16)(10000 - ((run-CORT_IdleUsec) * 10000) / USCNT_MAX);
		#undef  USCNT_MAX
		if (usage >= 0) 
		{
			CORT_CPUUsageVal = usage;
		}
		else
		{
			CORT_CPUUsageVal = 1;
		}
		CORT_IdleUsec = run;
#else
		CORT_EnterCritical();
		run = CORT_IdleCnt;
		CORT_IdleCnt = 0;
		CORT_ExitCritical();
		if (CORT_IdleCntMax > 0L)
		{
			usage = (run * 10000) / CORT_IdleCntMax;
			if (usage < 10000) 
			{
				CORT_CPUUsageVal = 10000 - usage;
			}
			else
			{
				CORT_CPUUsageVal = 1;
			}
		}
#endif
#if (CORT_HOOK_EN && CORT_HOOK_TASK_STATE_EN)
		if (CORT_Hook.hk_task_stat)
		{
			CORT_Hook.hk_task_stat();
		}
#endif
	}
}
#endif

/*
**
*/
#if (CORT_TIMER_EN || CORT_CALC_CPUUSAGE)
t_void CORT_HighTask(void *pd)
{
	while(1)
	{
#if (CORT_TIMER_EN)
		CORT_TimerProc();
#endif
#if (CORT_CALC_CPUUSAGE)
		CORT_StateProc();
#endif
        CORT_Task_Delay(1);
	}
}
#endif 

/*
**
*/
#if (CORT_SCHEDLOCK_EN)
void CORT_SchedLock(void)
{
	CORT_EnterCritical();
	if (CORT_Running)
	{
		if (CORT_LockCnt == 0 && CORT_LockCnt < (65535))
		{
			CORT_LockCnt++;
			CORT_OSStatus |= CORT_OS_STATUS_SCHEDLOCK;
		}
	}
	CORT_ExitCritical();	
}
#endif

/*
**
*/
#if (CORT_SCHEDLOCK_EN)
void  CORT_SchedUnLock(void)
{
	CORT_EnterCritical();
	if (CORT_Running)
	{
	    if (CORT_LockCnt > 0 && (--CORT_LockCnt) == 0)
		{
			CORT_OSStatus &= ~CORT_OS_STATUS_SCHEDLOCK;
			if (CORT_OSStatus == 0)
			{
				CORT_SearchRdy();
				
				if (CORT_CurPrio != CORT_HighPrio)
				{
					CORT_CtxSw();
				}
			}
	    }
	}
	CORT_ExitCritical();
}
#endif 

/*
**
*/
#if CORT_TASK_GETLIST_EN
t_tskid CORT_Task_GetNum(void)
{
	return CORT_AliveList.tl_cnt;
}
#endif

/*
**
*/
#if CORT_TASK_GETLIST_EN
t_u8 CORT_Task_GetNext(t_tskid CurID, t_tskid * pNextID)
{
	t_task * ptcb;
	t_tkitem * pitem;
#if (CORT_CHECK_PARAM)
	if (pNextID == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	if (CurID != (t_tskid)(-1) && CurID >= CORT_HeapNum)
	{
		return CORT_ERR_INVALID_PARAM;
	}

	CORT_EnterCritical();
	
	if (CurID == (t_tskid)(-1))
	{
		pitem = CORT_AliveList.tl_head;
	}
	else
	{
		ptcb  = &CORT_HeapFirst[CurID];
		pitem = ptcb->tk_list_task.ti_next;
	}

	while (pitem)
	{
		ptcb = pitem->ti_task;
#if (CORT_KERNEL_FULL && CORT_TASK_DEL_EN)
		if (ptcb->tk_alive_flag != CORT_ALIVE_FLAG_TASK)
		{
			pitem = ptcb->tk_list_task.ti_next;
			continue;
		}
#endif
		*pNextID = ptcb->tk_thread_id;
		CORT_ExitCritical();
		return CORT_OK;
	}

	CORT_ExitCritical();
	return CORT_ERR_NODATA;
}
#endif

/*
**
*/
t_void CORT_RTim_Run(void)
{
#if (CORT_HOOK_EN && CORT_TASK_RUNTIME_EN)
	if (CORT_Hook.hk_rtmcnt)
	{
		CORT_CurTcb->tk_rtim_cnt = CORT_Hook.hk_rtmcnt();
		CORT_CurTcb->tk_rtim_run  = m_true;
	}
#endif
}

/*
**
*/
t_void CORT_RTim_Stop(void)
{
#if (CORT_HOOK_EN && CORT_TASK_RUNTIME_EN)
	t_u32 us;
	t_u16 cnt = 0;
	
	if (CORT_Hook.hk_rtmcnt && CORT_CurTcb->tk_rtim_run)
	{
		CORT_CurTcb->tk_rtim_run = m_false;
		
		cnt = CORT_Hook.hk_rtmcnt();
		cnt = cnt - CORT_CurTcb->tk_rtim_cnt;
		us  = CORT_CurTcb->tk_rtim_usec + cnt;
		if (us >= ((t_u32)60*60*1000*1000))
		{
			CORT_CurTcb->tk_rtim_usec  = us - ((t_u32)60*60*1000*1000);
			CORT_CurTcb->tk_rtim_hour += 1;
		}
		else
		{
			CORT_CurTcb->tk_rtim_usec = us;
		}
	}
#endif
}

/*
**
*/
t_void  CORT_StateGo (void)
{
#if (CORT_CALC_CPUUSAGE)
#if (CORT_TASK_RUNTIME_EN)
	t_task * ptcb = &CORT_TaskHeap[CORT_IdleID];
	CORT_EnterCritical();
	ptcb->tk_rtim_hour = 0;
	ptcb->tk_rtim_usec = 0;
	CORT_StatusRdy = m_true;
    CORT_ExitCritical();
#else
    CORT_EnterCritical();
    CORT_IdleCnt = 0;
    CORT_ExitCritical();
    CORT_Task_Delay(CORT_TICK_PER_SEC / CORT_STAT_PRE_SEC);
    CORT_EnterCritical();
    CORT_IdleCntMax = CORT_IdleCnt;
	CORT_StatusRdy = m_true;
    CORT_ExitCritical();
#endif
#endif
}

/*
**
*/
t_void CORT_CreateOSTask(void)
{	
	CORT_Task_Create (CORT_IdleTask, 0, CORT_IdleTask_Stk, sizeof(CORT_IdleTask_Stk), CORT_LOWEST_PRIO, "Task Idle", &CORT_IdleID);
#if (CORT_TIMER_EN || CORT_CALC_CPUUSAGE)
	CORT_Task_Create (CORT_HighTask, 0, CORT_HighTask_Stk, sizeof(CORT_HighTask_Stk),                0, "Task High", m_null);
#endif
}

/*
**
*/
void CORT_Init(void)
{
	CORT_CommInit();
	CORT_CreateOSTask();
#if (CORT_DAI_EN)
	CORT_DAI_Init();
#endif
}

/*
**
*/
void CORT_Start(void)
{
	if (CORT_Running == m_false)
	{
		CORT_SearchRdy();
		CORT_StartHigh();
	}
}

/*
**
*/
#if (CORT_HOOK_EN)
void CORT_Hook_RegCall(t_hook * pHook)
{
	CORT_EnterCritical();
	if (pHook)
	{
		CORT_Hook = *pHook;
	}
	else
	{
		CORT_MemSet((t_void*)&CORT_Hook,0,sizeof(CORT_Hook));
	}
	CORT_ExitCritical();
}
#endif

/*
**
*/
t_u32 CORT_GetTickCounter(void)
{
	return CORT_TickCnt;
}


/*
**
*/
void CORT_EnterInt(void)
{
	CORT_OSStatus |=  CORT_OS_STATUS_INTERRUP;
}

/*
**
*/
void CORT_ExitInt(void)
{
	CORT_OSStatus &= ~CORT_OS_STATUS_INTERRUP;
}

/*
**
*/
t_u16 CORT_CpuUsage(void)
{
	return CORT_CPUUsageVal;
}

