/*
 * Copyright (C) Yao Jun 2011
 * -- nter@tom.com
 * -- http://CortOS.sourceforge.net/
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ****************************************
 * device application interface
 * ****************************************
 */

#ifndef   __CORTOS_DAI_H
#define   __CORTOS_DAI_H

#if (CORT_DAI_EN)

typedef struct t_device 	t_device;
typedef struct t_led_set 	t_led_set;
typedef struct t_i2c_set	t_i2c_set;
typedef struct t_i2c_opr 	t_i2c_opr;
typedef struct t_uart_set	t_uart_set;

/*******************************************************************************
** Define device type
*******************************************************************************/
#define CORT_DAIT_LED		0x00
#define CORT_DAIT_UART		0x01
#define CORT_DAIT_I2C		0x02

/*******************************************************************************
** Operating parameters for led device
*******************************************************************************/
typedef enum{
	eIOCtl_LED_SET		=	0x01
}eIOCtl_LED;

struct t_led_set{
	t_u32   mask;				/* led deivce : mask		    */
								/*              bit mask        */
	t_bool onoff;				/* led deivce : m_true :  on	*/
								/*              m_false: off 	*/
};

/*******************************************************************************
** Operating parameters for uart device
*******************************************************************************/
#define CORT_UART_BYTESIZE_5    0x00
#define CORT_UART_BYTESIZE_6    0x01
#define CORT_UART_BYTESIZE_7    0x02
#define CORT_UART_BYTESIZE_8    0X03

#define CORT_UART_PARITY_NONE   0x00
#define CORT_UART_PARITY_ODD    0x01
#define CORT_UART_PARITY_EVEN   0x03

#define CORT_UART_STOPBIT_1     0x00
#define CORT_UART_STOPBIT_1_5   0x01
#define CORT_UART_STOPBIT_2     0x02

typedef enum{
	eIOCtl_UART_SET		=	0x01
}eIOCtl_UART;

struct t_uart_set{
	t_u32 baud_rate;			/* Specify buad rate of communicaiton.        */
	t_u8  stop_bits;			/* Specify stopbit of communication.          */
	t_u8  Parity;				/* Specify parity of communication.           */
	t_u8  byte_size;			/* Specify byte of size of communication.     */
};
/*******************************************************************************
** Operating parameters for i2c device
*******************************************************************************/
typedef enum{
	eIOCtl_I2C_SET		=	0x01,
	eIOCtl_I2C_OPR		=	0x02
}eIOCtl_I2C;

struct t_i2c_set{
	t_u32  baud_rate;			/* i2c deivce : buad rate                     */
};
struct t_i2c_opr{
	t_u8   dev_addr;			/* i2c deivce : 8bit address for device 	  */

	t_u8   subreg_len;			/* i2c deivce : byte length for sub-register  */
	t_u16  subreg_val;			/* i2c deivce : value for sub-register  	  */
	t_bool subreg_bMSB;			/* i2c deivce : true : Big-Endian 			  */
								/* 				false: little-Endian		  */
	t_u16  write_len;			/* i2c deivce : byte lenght for write data 	  */
	t_u8 * write_buf;			/* i2c deivce : pointer for write data buffer */

  	/*|START | ADDR(W) | WRITE | STOP(endstop) | START | ADDR(R) | READ | STOP|*/
	
	t_bool write_endstop;		/* i2c deivce : send data end, then stop bus  */
	  
	t_u16  read_len;			/* i2c deivce : byte length for read data 	  */
	t_u8 * read_buf;			/* i2c deivce : pointer for read data buffer  */
};

/*******************************************************************************
** Device Node
*******************************************************************************/
struct t_device{
	t_u8    dev_type;
	t_u8    dev_idx;
	t_s8   (*open)(void);
#if (CORT_DAI_CLOSE)
	t_s8   (*close)(void);
#endif
#if (CORT_DAI_READ)	
	t_s32  (*read)(t_u8*,t_s32,t_s32);
#endif
#if (CORT_DAI_WRITE)
	t_s32  (*write)(t_u8*,t_s32,t_s32);
#endif
#if (CORT_DAI_IOCTRL)
	t_s8   (*ioctrl)(t_u8, t_handle,t_s32);
#endif
#if (CORT_DAI_STANDBY)
	t_s8   (*standby)(void);
	t_s8   (*resume)(void);
	t_bool (*allow_standby)(void);
#endif
	//Private
#if (CORT_DAI_READ_PROTECT_EN)
	t_sem   sem_read;
#endif
#if (CORT_DAI_WRITE_PROTECT_EN)
	t_sem   sem_write;
#endif
#if (CORT_DAI_IOCTRL_PROTECT_EN)
	t_sem	sem_ioctrl;
#endif
	t_device *next;
};

/*
** Init device application interface
*/
extern t_void    CORT_DAI_Init(void);
/*
** Register one device
*/
extern t_s8      CORT_DAI_RegDev (t_device * pDev);
/*
** Operation device
*/
extern t_device *CORT_DAI_Open   (t_u8 nDevType, t_u8 nDevIdx, t_s32 nTimOutTick);
extern t_s8      CORT_DAI_Close  (t_device * pDev, t_s32 nTimOutTick);
extern t_s32     CORT_DAI_Read   (t_device * pDev, t_u8 *pBuf, t_s32 nLen, t_s32 nTimOutTick);
extern t_s32     CORT_DAI_Write  (t_device * pDev, t_u8 *pBuf, t_s32 nLen, t_s32 nTimOutTick);
extern t_s8      CORT_DAI_IOCtrl (t_device * pDev, t_u8  nParam, t_handle hParam, t_s32 nTimOutTick);
extern t_s8      CORT_DAI_Standby(t_device * pDev, t_s32 nTimOutTick);
extern t_s8      CORT_DAI_Resume (t_device * pDev, t_s32 nTimOutTick);
extern t_bool    CORT_DAI_IsAllowStandby(t_device * pDev, t_s32 nTimOutTick);

#endif
#endif

