/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contents:
 */

static t_prtab  CORT_ReadyTab = {0};
static t_tklist CORT_DelayList = {0};
static t_tklist CORT_SuppendList = {0};

/*
**
*/
static void CORT_TableInsert(t_prtab * pTab, t_tkitem * pItem)
{
	t_tkitem **ppHead = &pTab->tb_head[pItem->ti_task->tk_priority];
	t_tkitem   *pHead = *ppHead;
	
	if (pHead)
	{
		/* Circular linked list */
		pItem->ti_prev = pHead->ti_prev;
		pItem->ti_next = pHead;
		pHead->ti_prev->ti_next = pItem;
		pHead->ti_prev = pItem;
	}
	else
	{
		pItem->ti_next = pItem;
		pItem->ti_prev = pItem;
		*ppHead        = pItem;
	}
	pTab->tb_prio_mask |= pItem->ti_task->tk_prio_mask;
}

/*
**
*/
static void CORT_TableRemove(t_prtab * pTab, t_tkitem * pItem)
{
	t_tkitem **ppHead = &pTab->tb_head[pItem->ti_task->tk_priority];
	t_tkitem   *pHead = *ppHead;

	if (pHead == pItem)
	{
		pHead = pHead->ti_next;
		*ppHead = pHead;
	}

	if (pHead == pItem)
	{
		*ppHead = m_null;
		pTab->tb_prio_mask &= ~pItem->ti_task->tk_prio_mask;
	}
	else
	{
		pItem->ti_prev->ti_next = pItem->ti_next;
		pItem->ti_next->ti_prev = pItem->ti_prev;
	}
}

/*
**
*/
static t_bool CORT_TaskDelWaitSem(t_task * ptcb)
{
	if (ptcb->tk_waitsem)
	{
		CORT_TableRemove(&ptcb->tk_waitsem->sm_wait_tab, &ptcb->tk_list_sem);
		ptcb->tk_waitsem = m_null;
		return m_true;
	}
	return m_false;
}

/*
**
*/
t_void CORT_SwitchTask(t_u8 prio)
{
	/* Switch to wait task */
	if ( CORT_Running && 
		!CORT_OSStatus && 
		 CORT_CurPrio > prio)
	{
		CORT_HighPrio = prio;
		CORT_CtxSw();
	}
}
/*
**
*/
t_task* CORT_GetHighTask(t_prtab * pTab, t_prio waittab, t_prio * RamainTab)
{
	t_u8 prio;
	t_task * ptcb;
	t_tkitem *pHead,*pItem;

	while(waittab)
	{
		prio  = (t_u8)CORT_CountLeadZeros(waittab);
		pHead = pTab->tb_head[prio];
		pItem = pHead;
		ptcb  = pItem->ti_task;
		waittab &= ~ptcb->tk_prio_mask;
		
		do{
			if (ptcb->tk_status == CORT_TK_STATUS_WAITING)
			{
				if (RamainTab)
				{
					*RamainTab = waittab;
				}
				return ptcb;
			}
			pItem = pItem->ti_next;
			ptcb = pItem->ti_task;
			waittab &= ~ptcb->tk_prio_mask;
		}while(pItem != pHead);
		
	}
	return m_null;
}

/*
**
*/
t_void CORT_ResumeTask(t_sem * psem, t_task *ptcb, t_bool bSwitch, t_s8 ret)
{
	t_prtab *pWaitTab = &psem->sm_wait_tab;

	if (ptcb == m_null)
	{
		ptcb = CORT_GetHighTask(pWaitTab, pWaitTab->tb_prio_mask, m_null);
	}
	if (ptcb != m_null)
	{
		if (ptcb->tk_delay_pbk)
		{
			*ptcb->tk_delay_pbk = 0;
			 ptcb->tk_delay_pbk = m_null;
		}		
		ptcb->tk_ret       = ret;
		ptcb->tk_waitsem   = m_null;
		ptcb->tk_delay_cnt = 0;
		ptcb->tk_status    = CORT_TK_STATUS_READY;
		
		CORT_TableRemove (pWaitTab,       &ptcb->tk_list_sem);
		CORT_ListRemove  (&CORT_DelayList,&ptcb->tk_list_nor);
		CORT_TableInsert (&CORT_ReadyTab, &ptcb->tk_list_nor);

		if (bSwitch)
		{
			CORT_SwitchTask(ptcb->tk_priority);
		}
	}
}

/*
**
*/
t_u8 CORT_TaskToWait(t_sem * psem, t_s32 * ptick, t_u8 m8, t_u32 m32)
{
	t_s32 tick = *ptick;
	
	if (tick == 0)
	{
		return CORT_ERR_TIMEOUT;
	}
	if (CORT_Running == m_false)
	{
		return CORT_ERR_NORUNING;
	}
	if (CORT_OSStatus != 0)
	{
		return CORT_ERR_INVALID_SUSPEND;
	}

	/* current task */
#if (CORT_FLAG_EN)
	CORT_CurTcb->tk_param_u8  = m8;
	CORT_CurTcb->tk_param_u32 = m32;
#endif
	CORT_CurTcb->tk_delay_pbk = ptick;
	CORT_CurTcb->tk_delay_cnt = tick;
	CORT_CurTcb->tk_waitsem   = psem;
	CORT_CurTcb->tk_ret       = CORT_ERROR;
	CORT_CurTcb->tk_status = CORT_TK_STATUS_WAITING;	
	
	CORT_TableRemove(&CORT_ReadyTab,   &CORT_CurTcb->tk_list_nor);
	CORT_ListInsert (&CORT_DelayList,  &CORT_CurTcb->tk_list_nor);
	CORT_TableInsert(&psem->sm_wait_tab,&CORT_CurTcb->tk_list_sem);
	
	CORT_SearchRdy();
	CORT_CtxSw();

	return CORT_CurTcb->tk_ret;
}

/*
**
*/
void CORT_SearchRdy(void)
{
	CORT_HighPrio = CORT_CountLeadZeros(CORT_ReadyTab.tb_prio_mask);
}

/*
**
*/
t_void CORT_Tick(void)
{
	t_task *ptcb;
	t_tkitem *pitem,*pcur;

#if (CORT_HOOK_EN && CORT_HOOK_TICK_EN)
	if (CORT_Hook.hk_tick)
	{
		CORT_Hook.hk_tick();
	}
#endif
	CORT_EnterCritical();
	CORT_TickCnt++;
	pitem = CORT_DelayList.tl_head;
	while (pitem) 
	{
		pcur   = pitem;
		pitem  = pitem->ti_next;
		ptcb   = pcur->ti_task;
		
		if ((ptcb->tk_delay_cnt == 0) ||
		    (ptcb->tk_delay_cnt > 0 && ((--ptcb->tk_delay_cnt)==0)))
		{
#if (CORT_EVENT_EN)
			if (CORT_TaskDelWaitSem(ptcb))
			{
				ptcb->tk_ret = CORT_ERR_TIMEOUT;
			}
#endif
			CORT_ListRemove (&CORT_DelayList, &ptcb->tk_list_nor);
			CORT_TableInsert(&CORT_ReadyTab,  &ptcb->tk_list_nor);

			ptcb->tk_status = CORT_TK_STATUS_READY;
		}
	}
	CORT_SearchRdy();
	CORT_ExitCritical();
}

/*
**
*/
t_void CORT_SwHighTask(void)
{
	t_tkitem **ppHead;
	
	if (!(CORT_OSStatus & CORT_OS_STATUS_SCHEDLOCK))
	{
#if (CORT_TASK_CHECK_STK_EN)
		CORT_CheckTaskStk(CORT_CurTcb);
#endif
		if (CORT_CurTcb && (CORT_CurTcb->tk_status == CORT_TK_STATUS_RUN))
		{
			CORT_CurTcb->tk_status = CORT_TK_STATUS_READY;
		}
		
		 ppHead = &CORT_ReadyTab.tb_head[CORT_HighPrio];
		*ppHead = (*ppHead)->ti_next;

#if (CORT_HOOK_EN && CORT_HOOK_TASK_SWITCH_EN)
		if (CORT_Hook.hk_task_switch)
		{
			CORT_Hook.hk_task_switch();
		}
#endif
		CORT_CurPrio = CORT_HighPrio;
		CORT_CurTcb  = CORT_ReadyTab.tb_head[CORT_HighPrio]->ti_task;
		
		CORT_CurTcb->tk_status = CORT_TK_STATUS_RUN;
#if CORT_TASK_SCHEDCNT_EN
		CORT_CurTcb->tk_sched_cnt += 1;
#endif
	}
}


