/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */

/*
** Create Flag
** Allowed From: Task Timer Interrupt
*/
#if (CORT_FLAG_EN)
t_s8  CORT_Flag_Create(t_flag *pflag, t_u32 init_val)
{
#if (CORT_CHECK_PARAM)
	if (pflag == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pflag->fg_alive_flag == CORT_ALIVE_FLAG_FLAG)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	pflag->fg_alive_flag = CORT_ALIVE_FLAG_FLAG;
#endif

	pflag->fg_flag = init_val;
	CORT_SemInit(&pflag->fg_sem, 0);

	CORT_ExitCritical();	
	return CORT_OK;
}
#endif 

/*
** Delete Flag
** Allowed From: Task
*/
#if (CORT_FLAG_EN && CORT_FLAG_DEL_EN)
t_s8  CORT_Flag_Delete(t_flag *pflag)
{
#if (CORT_CHECK_PARAM)
	if (pflag == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pflag->fg_alive_flag != CORT_ALIVE_FLAG_FLAG)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	pflag->fg_alive_flag = 0;
#endif

	CORT_DeleteSemaphore(&pflag->fg_sem, m_true);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif 

/*
** Obtain flag
** Allowed From: Task Timer Interrupt
*/
#if (CORT_FLAG_EN)
t_s8  CORT_Flag_Obtain(t_flag *pflag, t_u32 *pval, t_u32 mask, t_u8 oper, t_s32 tick)
{				   
	t_s8 ret;
	
#if (CORT_CHECK_PARAM)
	if (pflag == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pval == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
	if ((pval) == m_null ||
		(oper&0x7f) < CORT_SET_ANY ||
		(oper&0x7f) > CORT_CLR_ALL)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pflag->fg_alive_flag != CORT_ALIVE_FLAG_FLAG)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	while (CORT_OK != CORT_CheckFlag(pval, (t_u32*)&pflag->fg_flag, mask, oper))
	{
		ret = CORT_TaskToWait(&pflag->fg_sem, &tick, oper&0x7f, mask);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}

	CORT_ExitCritical();
    return CORT_OK;
}
#endif 

/*
** Set flag
** Allowed From: Task Timer Interrupt
*/
#if (CORT_FLAG_EN)
extern t_s8  CORT_Flag_Set(t_flag *pflag, t_u32 mask, t_u8 oper)
{
	t_prio waittab;
	t_sem * psem;
	t_task * ptcb;
	
#if (CORT_CHECK_PARAM)
	if (pflag == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if ((mask) == 0 ||
		(oper&0x7f) > CORT_NOR)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (pflag->fg_alive_flag != CORT_ALIVE_FLAG_FLAG)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	switch(oper)
	{
	case CORT_OR:  pflag->fg_flag |= mask; break;
	case CORT_AND: pflag->fg_flag &= mask; break;
	case CORT_NOR: pflag->fg_flag ^= mask; break;
	default:
		CORT_ExitCritical();
		return CORT_ERR_INVALID_PARAM;
	}

	psem = &pflag->fg_sem;

	waittab = psem->sm_wait_tab.tb_prio_mask;
	while(1)
	{
		ptcb = CORT_GetHighTask(&psem->sm_wait_tab, waittab, &waittab);
		if (ptcb == m_null)
		{
			break;
		}
		oper = ptcb->tk_param_u8;
		mask = ptcb->tk_param_u32;

		if (CORT_OK == CORT_CheckFlag(&ptcb->tk_param_u32, &pflag->fg_flag, mask, oper))
		{
			CORT_ResumeTask(psem, ptcb, m_true, CORT_OK);
			break;
		}
	}
	CORT_ExitCritical();
	return CORT_OK;
}
#endif 


