/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */


/*
**
*/
#if (CORT_EVENT_EN)
void CORT_SemInit(t_sem * psem, t_u32 init)
{
	CORT_MemSet(&psem->sm_wait_tab, 0,sizeof(t_prtab));

	psem->sm_cnt = init;
#if (CORT_CHECK_ALIVE)
	psem->sm_alive_flag = CORT_ALIVE_FLAG_SEM;
#endif
}
#endif
/*
**
*/
#if (CORT_EVENT_EN)
void CORT_SemDestroy(t_sem * psem)
{
#if (CORT_CHECK_ALIVE)
	psem->sm_alive_flag = CORT_ALIVE_FLAG_SEM;
#endif
}
#endif

/*
**
*/
t_void CORT_DeleteSemaphore(t_sem * psem, t_bool bSwHigh)
{
	t_prio *pProMask;
	
	/* Read Semaphore */
	pProMask = &psem->sm_wait_tab.tb_prio_mask;
	while(*pProMask)
	{
		CORT_ResumeTask(psem, m_null, m_false, CORT_ERR_INVALID_HANDLE);
	}

	CORT_SemDestroy(psem);

	if (bSwHigh)
	{
		CORT_SearchRdy();
		CORT_SwitchTask(CORT_HighPrio);
	}
}

/*
** Create semaphore
** Allowed From: Task Timer Interrupt
*/
#if (CORT_SEM_EN)
t_s8 CORT_Sem_Create(t_sem * psem, t_u32 init_val)
{	
#if (CORT_CHECK_PARAM)
	if (psem == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (psem->sm_alive_flag == CORT_ALIVE_FLAG_SEM)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	
	CORT_SemInit(psem, init_val);

	CORT_ExitCritical();
	return CORT_OK;	
}
#endif

/*
** Create semaphore
** Allowed From: Task Timer Interrupt
*/
#if (CORT_SEM_EN && CORT_SEM_DEL_EN)
t_s8  CORT_Sem_Delete(t_sem * psem)
{	
#if (CORT_CHECK_PARAM)
	if (psem == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (psem->sm_alive_flag != CORT_ALIVE_FLAG_SEM)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
	psem->sm_alive_flag = 0;
#endif

	CORT_DeleteSemaphore(psem, m_true);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Obtain semaphore
** Allowed From: Task Timer Interrupt
*/
#if (CORT_SEM_EN)
t_s8 CORT_Sem_Obtain(t_sem * psem, t_s32 tick)
{
	t_s8 ret;
#if (CORT_CHECK_PARAM)
	if (psem == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (psem->sm_alive_flag != CORT_ALIVE_FLAG_SEM)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	
	while(!psem->sm_cnt)
	{
		ret = CORT_TaskToWait(psem, &tick, 0, 0);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}
	psem->sm_cnt--;
		
	CORT_ExitCritical();
	return CORT_OK; 
}
#endif

/*
** Release semaphore
** Allowed From: Task Timer Interrupt
*/
#if (CORT_SEM_EN)
t_s8 CORT_Sem_Release(t_sem * psem)
{	
#if (CORT_CHECK_PARAM)
	if (psem == m_null)
	{
		return CORT_ERR_INVALID_HANDLE; 
	}
#endif
	CORT_EnterCritical();
#if (CORT_CHECK_ALIVE)
	if (psem->sm_alive_flag != CORT_ALIVE_FLAG_SEM)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif	

	psem->sm_cnt++;

	CORT_ResumeTask(psem, m_null, m_true, CORT_OK);	
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif


