/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */


static t_task  *CORT_PrioTab [CORT_LOWEST_PRIO + 1] = {0};

/*
**
*/
t_void CORT_SwitchTask(t_u8 prio)
{
	//Switch to wait task
	if ( CORT_Running && 
		!CORT_OSStatus && 
		 CORT_CurPrio > prio)
	{
		CORT_HighPrio = prio;
		CORT_CtxSw();
	}
}
/*
**
*/
t_task* CORT_GetHighTask(t_u32 waittab, t_prio * RamainTab)
{
	t_u8 prio;
	t_task *ptcb;

	while(waittab)
	{
		prio     = (t_u8)CORT_CountLeadZeros(waittab);
		ptcb     = CORT_PrioTab[prio];
		waittab &= ~ptcb->tk_prio_mask;

		if (ptcb->tk_status != CORT_TK_STATUS_WAITING)
		{
			continue;
		}
		if (RamainTab)
		{
			*RamainTab = waittab;
		}
		return ptcb;
	}
	return m_null;
}
/*
**
*/
#if CORT_EVENT_EN
void CORT_ResumeTask(t_sem * psem, t_task *ptcb)
{
	t_prio pmask;

	if (ptcb == m_null)
	{
		ptcb = CORT_GetHighTask(psem->sm_wait_mask, m_null);
	}
	if (ptcb)
	{
		pmask = ptcb->tk_prio_mask;
		psem->sm_wait_mask &= ~pmask;

		if(CORT_ReadyTable & pmask)
		{
			return;
		}
		CORT_ReadyTable  |=  pmask;

		if (ptcb->tk_delay_pbk)
		{
			*ptcb->tk_delay_pbk = 0;
			 ptcb->tk_delay_pbk = m_null;
		}
		ptcb->tk_delay_cnt = 0;
		ptcb->tk_waitsem   = m_null;
		ptcb->tk_ret       = CORT_OK;
		ptcb->tk_status    = CORT_TK_STATUS_READY;

		CORT_SwitchTask(ptcb->tk_priority);
	}
}
#endif
/*
**
*/
#if CORT_EVENT_EN
t_u8 CORT_TaskToWait(t_sem * psem, t_s32 * ptick, t_u8 m8, t_u32 m32)
{
	t_s32 tick = *ptick;
	t_prio priomask;
	
	if (tick == 0)
	{
		return CORT_ERR_TIMEOUT;
	}
	if (CORT_Running == m_false)
	{
		return CORT_ERR_NORUNING;
	}
	if (CORT_OSStatus != 0)
	{
		return CORT_ERR_INVALID_SUSPEND;
	}
	
	//current task
#if (CORT_FLAG_EN)
	CORT_CurTcb->tk_param_u8  = m8;
	CORT_CurTcb->tk_param_u32 = m32;
#endif
	CORT_CurTcb->tk_delay_pbk = ptick;
	CORT_CurTcb->tk_delay_cnt = tick;
	CORT_CurTcb->tk_waitsem   = psem;
	CORT_CurTcb->tk_ret       = CORT_ERROR;
	CORT_CurTcb->tk_status    = CORT_TK_STATUS_WAITING;
	
	priomask = CORT_CurTcb->tk_prio_mask;
	psem->sm_wait_mask |=  priomask;
	CORT_ReadyTable    &= ~priomask;

	CORT_SearchRdy();
	CORT_CtxSw();

	return CORT_CurTcb->tk_ret;
}
#endif
/*
**
*/
void CORT_SearchRdy(void)
{
	CORT_HighPrio = CORT_CountLeadZeros(CORT_ReadyTable);
}

/*
**
*/
void CORT_Tick(void)
{
	t_task *ptcb;
	t_tkitem *pitem;

#if (CORT_HOOK_EN && CORT_HOOK_TICK_EN)
	if (CORT_Hook.hk_tick)
	{
		CORT_Hook.hk_tick();
	}
#endif
	if (CORT_Running == m_false)
	{
		return;
	}
	CORT_EnterCritical();
	CORT_TickCnt++;
	pitem = CORT_AliveList.tl_head;
	while (pitem) 
	{
		ptcb   = pitem->ti_task;
		pitem  = pitem->ti_next;
		
		if (CORT_ReadyTable & ptcb->tk_prio_mask)
			continue;

		if (ptcb->tk_status != CORT_TK_STATUS_WAITING)
			continue;

		if ((ptcb->tk_delay_cnt == 0) ||
			(ptcb->tk_delay_cnt > 0 && (--ptcb->tk_delay_cnt) == 0))
		{
			if (ptcb->tk_delay_pbk)
			{
				*ptcb->tk_delay_pbk = 0;
				 ptcb->tk_delay_pbk = m_null;
			}
#if CORT_EVENT_EN
			if (ptcb->tk_waitsem)
			{
				ptcb->tk_waitsem->sm_wait_mask &= ~ptcb->tk_prio_mask;
				ptcb->tk_waitsem = m_null;
				ptcb->tk_ret = CORT_ERR_TIMEOUT;
			}
#endif
			ptcb->tk_status  = CORT_TK_STATUS_READY;
			CORT_ReadyTable |= ptcb->tk_prio_mask;
		}
	}
	CORT_SearchRdy();
	CORT_ExitCritical();
}

/*
**
*/
t_void CORT_SwHighTask(void)
{
	if (CORT_OSStatus == 0)
	{
#if (CORT_TASK_CHECK_STK_EN)
		CORT_CheckTaskStk(CORT_CurTcb);
#endif
		if (CORT_CurPrio != CORT_HighPrio)
		{
			if (CORT_CurTcb && (CORT_CurTcb->tk_status == CORT_TK_STATUS_RUN))
			{
				CORT_CurTcb->tk_status = CORT_TK_STATUS_READY;
			}
			
#if (CORT_HOOK_EN && CORT_HOOK_TASK_SWITCH_EN)
			if (CORT_Hook.hk_task_switch)
			{
				CORT_Hook.hk_task_switch();
			}
#endif
			CORT_CurPrio = CORT_HighPrio;
			CORT_CurTcb  = CORT_PrioTab[CORT_HighPrio];		

			CORT_CurTcb->tk_status = CORT_TK_STATUS_RUN;
			
#if CORT_TASK_SCHEDCNT_EN
			CORT_CurTcb->tk_sched_cnt += 1;
#endif
		}
	}
}

/*
**
*/
void CORT_TaskError(t_task * ptcb)
{
	ptcb->tk_status = CORT_TK_STATUS_ERROR;

	if (ptcb->tk_delay_pbk)
	{
		*ptcb->tk_delay_pbk = 0;
		 ptcb->tk_delay_pbk = m_null;
	}
#if CORT_EVENT_EN
	if (ptcb->tk_waitsem)
	{
		ptcb->tk_waitsem->sm_wait_mask &= ~ptcb->tk_prio_mask;
		ptcb->tk_waitsem = m_null;
		ptcb->tk_ret = CORT_ERR_STACKOVERFLOW;
	}
#endif

	CORT_ReadyTable &= ~ptcb->tk_prio_mask;
}

