/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */

/*
** Create Mailbox
** Allowed From: Task Timer Interrupt
*/
#if (CORT_MAIL_EN)
t_s8  CORT_Mailbox_Create(t_mailbox * pmail)
{
#if (CORT_CHECK_PARAM)
	if (pmail == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
#endif
	CORT_EnterCritical();

#if (CORT_CHECK_ALIVE)
	pmail->mb_alive_flag = CORT_ALIVE_FLAG_MAILBOX;
#endif

	pmail->mb_buffer = m_null;
	CORT_SemInit(&pmail->mb_write, 1);
	CORT_SemInit(&pmail->mb_read,  0);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Delete Mailbox
** Allowed From: Task Timer Interrupt
*/
#if (CORT_MAIL_EN && CORT_MAIL_DEL_EN)
t_s8  CORT_Mailbox_Delete(t_mailbox * pmail)
{
	return CORT_ERROR;
}
#endif

/*
** Write Mailbox
** Allowed From: Task Timer Interrupt
*/
#if (CORT_MAIL_EN)
t_s8  CORT_Mailbox_Write(t_mailbox * pmail, void * pbuf, t_s32 tick)
{
	t_s8 ret;
	t_sem * psem_w, *psem_r;

#if (CORT_CHECK_PARAM)
	if (pmail == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (pbuf == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();

#if (CORT_CHECK_ALIVE)
	if (pmail->mb_alive_flag != CORT_ALIVE_FLAG_MAILBOX)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_r = &pmail->mb_read;
	psem_w = &pmail->mb_write;
	
	while(!psem_w->sm_cnt)
	{
		ret = CORT_TaskToWait(psem_w, &tick, 0, 0);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}
	psem_w->sm_cnt = 0;
	psem_r->sm_cnt = 1;
	pmail->mb_buffer = pbuf;

	/* Check read thask */
	CORT_ResumeTask(psem_r, m_null);

	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Read Mailbox
** Allowed From: Task Timer Interrupt
*/
#if (CORT_MAIL_EN)
t_s8  CORT_Mailbox_Read(t_mailbox * pmail, void ** ppbuf, t_s32 tick)
{
	t_s8 ret;
	t_sem * psem_w, *psem_r;
	
#if (CORT_CHECK_PARAM)
	if (pmail == m_null)
	{
		return CORT_ERR_INVALID_HANDLE;
	}
	if (ppbuf == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();

#if (CORT_CHECK_ALIVE)
	if (pmail->mb_alive_flag != CORT_ALIVE_FLAG_MAILBOX)
	{
		CORT_ExitCritical();
		return CORT_ERR_INVALID_HANDLE;
	}
#endif

	psem_r = &pmail->mb_read;
	psem_w = &pmail->mb_write;
	
	while(!psem_r->sm_cnt)
	{
		ret = CORT_TaskToWait(psem_r, &tick, 0, 0);
		if (ret != CORT_OK)
		{
			CORT_ExitCritical();
			return ret;
		}
	}
	psem_r->sm_cnt = 0;
	psem_w->sm_cnt = 1;
	*ppbuf = pmail->mb_buffer;

	/* Check Write task */
	CORT_ResumeTask(psem_w, m_null);
	
	CORT_ExitCritical();
	return CORT_OK;
}
#endif 


