/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */


/*
** Create Task
** Allowed From: Task Timer Interrupt
*/
t_s8 CORT_Task_Create (void (*task_entry)(void *p_arg), void *parg, t_stk *pstk, 
		t_size stksize, t_u8 prio, t_u8 *pName, t_tskid *pThreadID)
{
	t_stk *psp;
	t_task *ptcb;
#if (CORT_CHECK_PARAM)
	if (prio > CORT_LOWEST_PRIO)
	{
		return CORT_ERR_INVALID_PRIO;
	}
	if (task_entry == m_null)
	{
		return CORT_ERR_INVALID_ENTRY;
	}
	if (pstk == m_null)
	{
		return CORT_ERR_INVALID_STACK;
	}
	if (stksize == 0)
	{
		return CORT_ERR_INVALID_SIZE;
	}
#endif

	CORT_EnterCritical();
	if ((CORT_PrioTab[prio] == m_null) && (CORT_FreeList.tl_cnt))
	{
		CORT_PrioTab[prio] = (t_task*)-1;
		ptcb = CORT_FreeList.tl_head->ti_task;
		CORT_ListRemove(&CORT_FreeList, &ptcb->tk_list_task);		
		CORT_ExitCritical();
	
#if (CORT_TASK_CHECK_STK_EN)
		CORT_ClearStk(pstk, stksize);
#endif
		psp = CORT_InitStk (task_entry, parg, pstk, stksize);

		ptcb->tk_stack_top = psp;
#if (CORT_TASK_NAME_EN)
		ptcb->tk_name = pName;
#endif
	    ptcb->tk_delay_cnt = 0;
		ptcb->tk_delay_pbk = m_null;
		ptcb->tk_prio_mask = 1<<prio;
		ptcb->tk_priority = prio;
#if CORT_FLAG_EN
		ptcb->tk_param_u8 = 0;
		ptcb->tk_param_u32 = 0;
#endif
		ptcb->tk_status = CORT_TK_STATUS_READY;
#if CORT_EVENT_EN
		ptcb->tk_waitsem = m_null;
		ptcb->tk_ret = CORT_OK;
#endif
#if CORT_TASK_SCHEDCNT_EN
		ptcb->tk_sched_cnt = 0;
#endif
#if (CORT_TASK_CHECK_STK_EN)
		ptcb->tk_stack_free = stksize;
		ptcb->tk_stack_size = stksize;
		ptcb->tk_stack_bottom = pstk;
#endif
#if (CORT_TASK_RUNTIME_EN)
		ptcb->tk_rtim_run  = m_false;
		ptcb->tk_rtim_hour = 0;
		ptcb->tk_rtim_usec = 0;
		ptcb->tk_rtim_cnt  = 0;
#endif
		if (pThreadID)
		{
			*pThreadID = ptcb->tk_thread_id;
		}
#if (CORT_HOOK_EN && CORT_HOOK_TASK_CREATE_EN)
		if (CORT_Hook.hk_task_create)
		{
			CORT_Hook.hk_task_create(ptcb);
		}
#endif

		CORT_EnterCritical();

		CORT_PrioTab[prio] = ptcb;
		CORT_ReadyTable   |= ptcb->tk_prio_mask;
		CORT_ListInsert (&CORT_AliveList, &ptcb->tk_list_task);

		CORT_SwitchTask(prio);

		CORT_ExitCritical();
		return CORT_OK;
	}
    CORT_ExitCritical();
    return CORT_ERROR;
}

/*
** Delete Task
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TASK_DEL_EN)
t_s8  CORT_Task_Delete(t_tskid ThreadID)
{
	return CORT_ERROR;
}
#endif

/*
** Suspend Task
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TASK_SUSPEND_EN)
t_s8  CORT_Task_Suspend(t_tskid ThreadID)
{
	t_task * ptcb;

#if (CORT_CHECK_PARAM)
	if (ThreadID >= CORT_HeapNum)
	{
		return CORT_ERR_INVALID_THREADID;
	}
#endif
	CORT_EnterCritical();
	ptcb = &CORT_HeapFirst[ThreadID];
	if (ptcb->tk_status != CORT_TK_STATUS_SUSPEND &&
		ptcb->tk_status != CORT_TK_STATUS_ERROR)
	{
		ptcb->tk_status = CORT_TK_STATUS_SUSPEND;

		CORT_ReadyTable &= ~ptcb->tk_prio_mask;

		if (CORT_CurTcb == ptcb &&
			CORT_Running != m_false &&
			CORT_OSStatus == 0)
		{
			CORT_SearchRdy();
			CORT_CtxSw();
		}
	}
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Resume Task
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TASK_SUSPEND_EN)
t_s8  CORT_Task_Resume(t_tskid ThreadID)
{
	t_task * ptcb;

#if (CORT_CHECK_PARAM)
	if (ThreadID >= CORT_HeapNum)
	{
		return CORT_ERR_INVALID_THREADID;
	}
#endif
	CORT_EnterCritical();
	ptcb = &CORT_HeapFirst[ThreadID];
	if ((ptcb->tk_status == CORT_TK_STATUS_SUSPEND))
	{
		if (ptcb->tk_delay_cnt == 0)
		{
			CORT_ReadyTable |= ptcb->tk_prio_mask;
			ptcb->tk_status = CORT_TK_STATUS_READY;

			CORT_SwitchTask(ptcb->tk_priority);
		}
		else
		{
			ptcb->tk_status = CORT_TK_STATUS_WAITING;
		}
	}
	CORT_ExitCritical();
	return CORT_OK;
}
#endif

/*
** Get ID of Task
** Allowed From: Task Timer Interrupt
*/
#if (CORT_TASK_GETID_EN)
t_s8 CORT_Task_GetID(t_tskid *pThreadID)
{
#if (CORT_CHECK_PARAM)
	if (pThreadID == m_null)
	{
		return CORT_ERROR;
	}
#endif

	CORT_EnterCritical();
	if ((CORT_OSStatus & CORT_OS_STATUS_TIMING) == 0 &&
		(CORT_OSStatus & CORT_OS_STATUS_INTERRUP) == 0 &&
		(CORT_CurTcb != m_null))
	{
		*pThreadID = CORT_CurTcb->tk_thread_id;
		CORT_ExitCritical();
		return CORT_OK;
	}
	CORT_ExitCritical();
	return CORT_ERROR;
}
#endif

/*
** Get state for task
** Allowed From: 
*/
#if (CORT_TASK_GETSTAT_EN)
t_s8  CORT_Task_GetState(t_tskid ThreadID, t_tasksta * pState)
{
	t_task * ptcb;
#if (CORT_CHECK_PARAM)
	if (pState == m_null)
	{
		return CORT_ERR_INVALID_PARAM;
	}
	if (ThreadID >= CORT_HeapNum)
	{
		return CORT_ERR_INVALID_PARAM;
	}
#endif
	CORT_EnterCritical();
	ptcb = &CORT_HeapFirst[ThreadID];
	if (ptcb != m_null && ptcb->tk_stack_top)
	{
#if (CORT_TASK_NAME_EN)
		pState->ts_name = ptcb->tk_name;
#endif
#if (CORT_TASK_CHECK_STK_EN)
		CORT_CalcTaskStk(ptcb);
		pState->ts_stack_size = ptcb->tk_stack_size;
		pState->ts_stack_free = ptcb->tk_stack_free;
#endif
#if (CORT_TASK_SCHEDCNT_EN)
		pState->ts_sched_cnt = ptcb->tk_sched_cnt;
#endif
#if CORT_TASK_RUNTIME_EN
		pState->ts_rtim_hour = ptcb->tk_rtim_hour;
		pState->ts_rtim_usec = ptcb->tk_rtim_usec;
#endif
		CORT_ExitCritical();
		return CORT_OK;
	}
	CORT_ExitCritical();
	return CORT_ERR_INVALID_THREADID;
}
#endif


/*
** Delay
** Allowed From: Task
*/
void CORT_Task_Delay(t_s32 tick)
{
	if (tick < 0)
	{
		return;
	}

	CORT_EnterCritical();
	if (CORT_CurTcb != m_null &&
		CORT_Running != m_false &&
		CORT_OSStatus == 0)
	{
		CORT_CurTcb->tk_delay_pbk = m_null;
		CORT_CurTcb->tk_delay_cnt = tick;
		CORT_CurTcb->tk_status    = CORT_TK_STATUS_WAITING;
		CORT_ReadyTable          &= ~CORT_CurTcb->tk_prio_mask;
		CORT_SearchRdy();
		CORT_CtxSw();
	}
	CORT_ExitCritical();
}

