/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef   __CORTOS_TYPE_H
#define   __CORTOS_TYPE_H

/* Define service completion status constants.  */
#define CORT_OK                     0
#define CORT_ERROR                  1
#define CORT_ERR_NORUNING           2
#define CORT_ERR_TIMEOUT            3
#define CORT_ERR_NO_MORE_TCB        4
#define CORT_ERR_MAIL_FULL          5
#define CORT_ERR_MAIL_EMPTY         6
#define CORT_ERR_QUEUE_FULL         7
#define CORT_ERR_QUEUE_EMPTY        8
#define CORT_ERR_NOT_PRESENT        9
#define CORT_ERR_PIPE_FULL          10
#define CORT_ERR_PIPE_EMPTY         16
#define CORT_ERR_INVALID_HANDLE     17
#define CORT_ERR_INVALID_PRIO       18
#define CORT_ERR_INVALID_ENTRY      19
#define CORT_ERR_INVALID_STACK      20
#define CORT_ERR_INVALID_SIZE       21
#define CORT_ERR_INVALID_THREADID   22
#define CORT_ERR_INVALID_PARAM      23
#define CORT_ERR_INVALID_SUSPEND    24
#define CORT_ERR_NODATA				32
#define CORT_ERR_STACKOVERFLOW		33
#define CORT_SEMAPHORE_DELETED      70
#define CORT_QUEUE_DELETED          71
#define CORT_TIMER_DELETED          72
#define CORT_MAILBOX_DELETED        73
#define CORT_FLAG_DELETED           74
#define CORT_PIPE_DELETED           75



/* Define constants for use in service parameters.  */
#define CORT_OR						0x00
#define CORT_AND					0x01
#define CORT_NOR					0x02
#define CORT_SET_ANY				0x03
#define	CORT_SET_ALL				0x04
#define CORT_CLR_ANY				0x05
#define CORT_CLR_ALL				0x06
#define CORT_CONSUME				0x80

/* Define task suspension constants.  */
#define CORT_TASK_READY             0x00  /* Ready to execute. */
#define CORT_TASK_RUNNING           0x01  /* Running */
#define CORT_TASK_SLEEP             0x02  /* Sleeping... */
#define CORT_TASK_FINISHED          0x03  /* Returned from the entry function. */
#define CORT_TASK_TERMINATED        0x04  /* Terminated... */
#define CORT_TASK_PURE_SUSPEND      0x05  /* Unconditionally suspended */
#define CORT_TASK_MEMORY_SUSPEND    0x86  /* Suspended on a memory. */
#define CORT_TASK_SEMAPHORE_SUSPEND 0x87  /* Suspended on a semaphore. */
#define CORT_TASK_QUEUE_SUSPEND     0x88  /* Suspended on a queue. */
#define CORT_TASK_PIPE_SUSPEND      0x89  /* Suspended on a pipe. */
#define CORT_TASK_MAILBOX_SUSPEND   0x8a  /* Suspended on a mailbox. */
#define CORT_TASK_FLAG_SUSPEND	    0x8b  /* Suspended on a flag. */

/* Define Alive Flag */
#define CORT_ALIVE_FLAG_TASK		0xAF00
#define CORT_ALIVE_FLAG_SEM			0xAF11
#define CORT_ALIVE_FLAG_QUEUE		0xAF22
#define CORT_ALIVE_FLAG_PIPE		0xAF33
#define CORT_ALIVE_FLAG_MAILBOX		0xAF44
#define CORT_ALIVE_FLAG_FLAG		0xAF55
#define CORT_ALIVE_FLAG_TIMER		0xAF66

/* ************************************************************************** */
#define CORT_PRIO_NUM				(CORT_LOWEST_PRIO+1)

/* ************************************************************************** */
#define CORT_CALC_CPUUSAGE			(CORT_TASK_GETSTAT_EN && (!CORT_HOOK_SLEEP_EN || (CORT_TASK_RUNTIME_EN)))

/* ************************************************************************** */

#define	m_true						1
#define m_false 					0
#define	m_null						((t_void*)0)

#define CORT_MAX(a,b)				(((a)>(b)) ? (a) : (b))
#define CORT_MIN(a,b)				(((a)>(b)) ? (b) : (a))
#define CORT_EVENT_EN				(CORT_SEM_EN || CORT_QUEUE_EN || CORT_PIPE_EN || CORT_MAIL_EN || CORT_FLAG_EN)
#define CORT_ARRAYSIZE(a)			(sizeof(a) / sizeof((a)[0]))

typedef struct t_task      t_task;
typedef struct t_sem       t_sem;
typedef struct t_queue     t_queue;
typedef struct t_timer     t_timer;
typedef struct t_mailbox   t_mailbox;
typedef struct t_flag      t_flag;
typedef struct t_pipe      t_pipe;
typedef struct t_tasksta   t_tasksta;
typedef struct t_hook      t_hook;
typedef struct t_prtab     t_prtab;
typedef struct t_tkitem    t_tkitem;
typedef struct t_tklist    t_tklist;
typedef t_void*            t_handle;

typedef void (*t_timproc)(t_u8,t_bool);
 
/* Define Item of task link structure */ 
struct t_tkitem{
	t_task   *ti_task;
	t_tkitem *ti_next;
	t_tkitem *ti_prev;
};

/* Define Item of task link structure */ 
struct t_tklist{
	t_tskid   tl_cnt;
	t_tkitem *tl_head;
	t_tkitem *tl_tail;
};

/* Define Priority list structure */ 
#if (CORT_KERNEL_FULL)
struct t_prtab{
	t_prio   tb_prio_mask;
	t_tkitem *tb_head[CORT_LOWEST_PRIO + 1];
};
#endif

/* Define task control data structure */ 
struct t_task{
	t_stk   *tk_stack_top;
	t_s32    tk_delay_cnt;
	t_s32   *tk_delay_pbk;
	t_tskid  tk_thread_id;
	t_prio   tk_prio_mask;
	t_u8     tk_status;
	t_u8     tk_priority;
	t_u8     tk_ret;
	t_tkitem tk_list_task;
#if (CORT_TASK_NAME_EN)
	t_u8    *tk_name;
#endif
#if (CORT_KERNEL_FULL && CORT_TASK_DEL_EN)
	t_u16    tk_alive_flag;
#endif
#if (CORT_KERNEL_FULL && CORT_EVENT_EN)
	t_tkitem tk_list_sem;
#endif
#if (CORT_KERNEL_FULL)
	t_tkitem tk_list_nor;
#endif
#if (CORT_EVENT_EN)
	t_sem   *tk_waitsem;
#endif
#if (CORT_TASK_SCHEDCNT_EN)
	t_u32    tk_sched_cnt;
#endif
#if (CORT_TASK_CHECK_STK_EN)
	t_size   tk_stack_free;
	t_size   tk_stack_size;
	t_stk	*tk_stack_bottom;
#endif
#if (CORT_FLAG_EN)
	t_u32    tk_param_u32;
	t_u8     tk_param_u8;
#endif
#if CORT_TASK_RUNTIME_EN
	t_bool   tk_rtim_run;
	t_u16    tk_rtim_cnt;		/* Microseconds */
	t_u16    tk_rtim_hour;		/* Hour */
	t_u32    tk_rtim_usec;		/* Microseconds */
#endif
};

/* Define task state data structure */ 
#if (CORT_TASK_GETSTAT_EN)
struct t_tasksta{
	t_u8     ts_status;
#if (CORT_TASK_NAME_EN)
	t_u8    *ts_name;
#endif
#if (CORT_TASK_CHECK_STK_EN)
	t_size   ts_stack_free;
	t_size   ts_stack_size;
#endif
#if (CORT_TASK_SCHEDCNT_EN)
	t_u32    ts_sched_cnt;
#endif
#if (CORT_TASK_RUNTIME_EN)
	t_u16    ts_rtim_hour;		/* Hour */
	t_u32    ts_rtim_usec;		/* Microseconds */
#endif
};
#endif

/* Define semahore control data structure */ 
#if (CORT_EVENT_EN)
struct t_sem{
#if (CORT_CHECK_ALIVE)
	t_u16    sm_alive_flag;
#endif
#if (CORT_KERNEL_FULL)
	t_prtab  sm_wait_tab;
#else
	t_u32    sm_wait_mask;
#endif
	t_u32	 sm_cnt;
};
#endif

/* Define queue control data structure */ 
#if (CORT_QUEUE_EN)
struct t_queue{
#if (CORT_CHECK_ALIVE)
	t_u16    qu_alive_flag;
#endif
	t_sem    qu_write;
	t_sem    qu_read;
	t_void **qu_buffer;
	t_u16    qu_front;
	t_u16    qu_rear;
	t_u16    qu_num;
};
#endif

/* Define pipe control data structure */ 
#if (CORT_PIPE_EN)
struct t_pipe{
# if (CORT_CHECK_ALIVE)
	t_u16    pe_alive_flag;
#endif
	t_sem    pe_write;
	t_sem    pe_read;
	t_u8    *pe_buffer;
	t_s32    pe_front;
	t_s32    pe_rear;
	t_s32    pe_num;
	t_task  *pe_wtask;
};
#endif

/* Define mailbox control data structure */ 
#if (CORT_MAIL_EN)
struct t_mailbox{
#if (CORT_CHECK_ALIVE)
	t_u16   mb_alive_flag;
#endif
	t_sem   mb_write;
	t_sem   mb_read;
	t_void *mb_buffer;
};
#endif

/* Define flag control data structure */ 
#if (CORT_FLAG_EN)
struct t_flag{
#if (CORT_CHECK_ALIVE)
	t_u16   fg_alive_flag;
#endif
	t_sem	fg_sem;
	t_u32	fg_flag;
};
#endif

/* Define timer control data structure */ 
struct t_timer{
#if (CORT_CHECK_ALIVE)
	t_u16     tm_alive_flag;
#endif
	t_u8      tm_flag;
	t_s32     tm_loopcnt;
	t_u32     tm_remain;
	t_u32     tm_reload;
	t_timer * tm_next;
#if (CORT_TIMER_DEL_EN)
	t_timer * tm_prev;
#endif
	t_timproc tm_func;
};

/* Define Hook data structure */ 
#if (CORT_HOOK_EN)
struct t_hook{
	void    (*hk_task_create)(t_task *ptcb);
	void    (*hk_task_delete)(t_task *ptcb);
	void    (*hk_task_stat)(void);
	void    (*hk_task_switch)(void);
	void	(*hk_task_error)(t_tskid threadid, t_u8 errcode);
	t_u16   (*hk_rtmcnt)(void);
	void   	(*hk_sleep)(void);
	void    (*hk_tick)(void);
};
#endif

#endif

