#include <stdarg.h>


#define vaStart(list, param) 	list = (char*)((t_s32)&param + sizeof(param))
#define vaArg(list, type) 		((type *)(list += sizeof(type)))[-1]

static t_device * pDevUart = m_null;

void CORT_Printf_Init(void)
{
	pDevUart = CORT_DAI_Open(CORT_DAIT_UART, 1, -1);

	
}

void CORT_Printf_WH(t_u8 chr)
{
	CORT_DAI_Write(pDevUart, &chr, 1, -1);
}

void CORT_Printf_PutChr(t_u8 ucCh)
{
	if (ucCh == '\n')
	{
		CORT_Printf_WH('\r');
	}
	CORT_Printf_WH(ucCh);
}

void CORT_Printf_PutStr(t_s8 *string)
{
	while (*string != '\0')
	{
		CORT_Printf_PutChr(*string);
		string++;
	}
}


void CORT_Printf_PutRepChar(t_s8 c, t_s32 count)
{
	while (count--)
	CORT_Printf_PutChr(c);
}


void CORT_Printf_PutStringReverse(t_s8 *s, t_s32 index)
{
	while ((index--) > 0)
	CORT_Printf_PutChr(s[index]);
}


void CORT_Printf_PutNumber(t_s32 value, t_s32 radix, t_s32 width, t_s8 fill)
{
	t_s8    buffer[40];
	t_s32     bi = 0;
	t_u32  uvalue;
	t_u32  digit;
	t_u32  left = FALSE;
	t_u32  negative = FALSE;

	if (fill == 0)
	    	fill = ' ';

	if (width < 0)
	{
		width = -width;
		left = TRUE;
	}

	if (width < 0 || width > 80)
	    	width = 0;

	if (radix < 0)
	{
		radix = -radix;
		if (value < 0)
		{
			negative = TRUE;
			value = -value;
	    	}
	}

	uvalue = value;

	do
	{
		if (radix != 16)
		{
			digit = uvalue % radix;
			uvalue = uvalue / radix;
		}
		else
		{
			digit = uvalue & 0xf;
			uvalue = uvalue >> 4;
		}
		buffer[bi] = digit + ((digit <= 9) ? '0' : ('A' - 10));
		bi++;

		if (uvalue != 0)
		{
			if ((radix == 10)
			    && ((bi == 3) || (bi == 7) || (bi == 11) | (bi == 15)))
			{
				buffer[bi++] = ',';
			}
		}
	}
	while (uvalue != 0);

	if (negative)
	{
		buffer[bi] = '-';
		bi += 1;
	}

	if (width <= bi)
		CORT_Printf_PutStringReverse(buffer, bi);
	else
	{
		width -= bi;
		if (!left)
			CORT_Printf_PutRepChar(fill, width);
		CORT_Printf_PutStringReverse(buffer, bi);
		if (left)
		    	CORT_Printf_PutRepChar(fill, width);
	}
}


char *CORT_Printf_FormatItem(char *f, t_s32 a)
{
	char   c;
	t_s32    fieldwidth = 0;
	t_s32    leftjust = FALSE;
	t_s32    radix = 0;
	char   fill = ' ';

	if (*f == '0')
		fill = '0';

	while ((c = *f++) != 0)
	{
		if (c >= '0' && c <= '9')
		{
			fieldwidth = (fieldwidth * 10) + (c - '0');
		}
		else
			switch (c)
			{
				case '\000':
					return (--f);
				case '%':
				    	CORT_Printf_PutChr('%');
				    	return (f);
				case '-':
				    	leftjust = TRUE;
				    	break;
				case 'c':
				{
				        if (leftjust)
				        	CORT_Printf_PutChr(a & 0x7f);

				        if (fieldwidth > 0)
				            	CORT_Printf_PutRepChar(fill, fieldwidth - 1);

				        if (!leftjust)
				            	CORT_Printf_PutChr(a & 0x7f);
				        return (f);
				}
				case 's':
				{
				        if (leftjust)
				        	CORT_Printf_PutStr((t_s8*)a);

				        if (fieldwidth > strlen((char*)a))
				            	CORT_Printf_PutRepChar(fill, fieldwidth - strlen((char*)a));

				        if (!leftjust)
				           	CORT_Printf_PutStr((t_s8*)a);
				        return (f);
				}
				case 'd':
				case 'i':
				   	 radix = -10;
				break;
				case 'u':
				    	radix = 10;
				break;
				case 'x':
				    	radix = 16;
				break;
				case 'X':
				    	radix = 16;
				break;
				case 'o':
				    	radix = 8;
				break;
				default:
				    	radix = 3;
				break;      /* unknown switch! */
			}
		if (radix)
		    break;
	}

	if (leftjust)
	    	fieldwidth = -fieldwidth;

	CORT_Printf_PutNumber(a, radix, fieldwidth, fill);

	return (f);
}

void CORT_Printf(char* fmt,...)
{
	char *argP;

	vaStart(argP, fmt);       /* point at the end of the format string */
	while (*fmt)
	{                       /* this works because args are all ints */
		if (*fmt == '%')
		{
			fmt =
			CORT_Printf_FormatItem(
			fmt + 1, 
			vaArg(argP, t_s32));
		}
		else
		{
			CORT_Printf_PutChr(*fmt++);
		}
	}
}


