/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/* Includes ------------------------------------------------------------------*/
#include "config.h"
#include "test_comm.c"

/* Private define ------------------------------------------------------------*/
#define TASK_STK_SIZE_MAIN	128
#define TASK_STK_SIZE_I2C	128
#define TASK_PRIO_MAIN		1
#define TASK_PRIO_I2C		2

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
t_stk STK_TASK_MAIN[TASK_STK_SIZE_MAIN];
t_stk STK_TASK_I2C[TASK_STK_SIZE_I2C];


/*******************************************************************************
* Function Name  : I2CTask
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void I2CTask(void *pd)
{
	t_u8 val[2] = {0};
	t_device * pDev;
	t_i2c_set  i2cset;
	t_i2c_opr  i2copr;
		
	pDev = CORT_DAI_Open(CORT_DAIT_I2C, 1, -1);
	if (pDev)
	{
		i2cset.baud_rate = 100000;	//100khz
		CORT_DAI_IOCtrl (pDev, eIOCtl_I2C_SET, &i2cset, -1);
	}
	
	//EEPROM AT24C16
	i2copr.dev_addr		= 0xa0;			/* i2c deivce : 8bit address for device 	  */
	
	i2copr.subreg_len	= 1;			/* i2c deivce : byte length for sub-register  */
	i2copr.subreg_val	= 0x00;			/* i2c deivce : value for sub-register  	  */
	i2copr.subreg_bMSB	= m_false;		/* i2c deivce : true : Big-Endian 			  */
										/* 				false: little-Endian		  */
	i2copr.write_len	= 0;			/* i2c deivce : byte lenght for write data 	  */
	i2copr.write_buf	= m_null;		/* i2c deivce : pointer for write data buffer */
	i2copr.write_endstop= m_false;		/* i2c deivce : send data end, then stop bus  */
	
	i2copr.read_len		= 2;			/* i2c deivce : byte length for read data 	  */
	i2copr.read_buf		= val;			/* i2c deivce : pointer for read data buffer  */
	
	while(1)
	{
		if (CORT_OK == CORT_DAI_IOCtrl (pDev, eIOCtl_I2C_OPR, &i2copr, 50))
		{
			CORT_Printf("I2C Read AT24C16 :%02X %02X\n", val[0], val[1]);
		}
		else
		{
			CORT_Printf("I2C Read AT24C16 Error\n");
		}
		
		CORT_Task_Delay(500);
	}
}
/*******************************************************************************
* Function Name  : MainTask
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void MainTask(void *pd)
{
	dev_i2c_1_init();
	dev_hwinit_start();
	
	CORT_StateGo();

	CORT_Printf_Init();
	
	CORT_Task_Create (I2CTask, 0, STK_TASK_I2C, sizeof(STK_TASK_I2C), TASK_PRIO_I2C, "I2CTask", m_null);

	while(1)
	{
		CORT_Task_Delay(1);
	}
}
/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	dev_hwinit_init();
	
	CORT_Init();
	CORT_Task_Create (MainTask, 0, STK_TASK_MAIN, sizeof(STK_TASK_MAIN), TASK_PRIO_MAIN, "MainTask", m_null);
	CORT_Start();
}
