/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/* Includes ------------------------------------------------------------------*/
#include "config.h"
/* Private define ------------------------------------------------------------*/
#define TASK_STK_SIZE_MAIN	128
#define TASK_STK_SIZE_LED	128
#define TASK_PRIO_MAIN		1
#define TASK_PRIO_LED1		2
#define TASK_PRIO_LED2		3

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
t_stk STK_TASK_MAIN[TASK_STK_SIZE_MAIN];
t_stk STK_TASK_LED1[TASK_STK_SIZE_LED];
t_stk STK_TASK_LED2[TASK_STK_SIZE_LED];


/*******************************************************************************
* Function Name  : LedTask1
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LedTask1(void *pd)
{
	t_device * pDev;
	t_led_set  led;
		
	pDev = CORT_DAI_Open(CORT_DAIT_LED, 0, -1);
	while(1)
	{
		led.mask  = 0x0005;
		led.onoff = m_true;
		CORT_DAI_IOCtrl (pDev, eIOCtl_LED_SET, &led, -1);
		CORT_Task_Delay(CORT_TICK_PER_SEC/4);
		
		led.mask  = 0x0005;
		led.onoff = m_false;
		CORT_DAI_IOCtrl (pDev, eIOCtl_LED_SET, &led, -1);
		CORT_Task_Delay(CORT_TICK_PER_SEC/4);
	}
}
/*******************************************************************************
* Function Name  : LedTask1
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void LedTask2(void *pd)
{
	t_device * pDev;
	t_led_set  led;

	pDev = CORT_DAI_Open(CORT_DAIT_LED, 0, -1);
	while(1)
	{
		led.mask  = 0x000a;
		led.onoff = m_false;
		CORT_DAI_IOCtrl (pDev, eIOCtl_LED_SET, &led, -1);
		CORT_Task_Delay(CORT_TICK_PER_SEC/4);
		
		led.mask  = 0x000a;
		led.onoff = m_true;
		CORT_DAI_IOCtrl (pDev, eIOCtl_LED_SET, &led, -1);
		CORT_Task_Delay(CORT_TICK_PER_SEC/4);
	}
}
/*******************************************************************************
* Function Name  : MainTask
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void MainTask(void *pd)
{
	dev_led_0_init();
	dev_hwinit_start();
	
	CORT_StateGo();
	
	CORT_Task_Create (LedTask1, 0, STK_TASK_LED1, sizeof(STK_TASK_LED1), TASK_PRIO_LED1, "LedTask1", m_null);
	CORT_Task_Create (LedTask2, 0, STK_TASK_LED2, sizeof(STK_TASK_LED2), TASK_PRIO_LED2, "LedTask2", m_null);
	while(1)
	{
		CORT_Task_Delay(1);
	}
}
/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	dev_hwinit_init();
	
	CORT_Init();
	CORT_Task_Create (MainTask, 0, STK_TASK_MAIN, sizeof(STK_TASK_MAIN), TASK_PRIO_MAIN, "MainTask", m_null);
	CORT_Start();
}
