/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/* Includes ------------------------------------------------------------------*/
#include "config.h"
/* Private define ------------------------------------------------------------*/
#define TASK_STK_SIZE_MAIN	128
#define TASK_STK_SIZE_LED	128
#define TASK_PRIO_MAIN		1
#define TASK_PRIO_SEND		2

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
t_stk STK_TASK_MAIN[TASK_STK_SIZE_MAIN];
t_stk STK_TASK_SEND[TASK_STK_SIZE_LED];


/*******************************************************************************
* Function Name  : LedTask1
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void UartSendTask(void *pd)
{
	t_device * pDev;
	t_u8 * hstr = "Hello CortOS\r\n";
	t_u8 len;
	
	len = (t_u8)strlen((char*)hstr);
	
	pDev = CORT_DAI_Open(CORT_DAIT_UART, 1, -1);
	while(1)
	{
		CORT_DAI_Write(pDev, hstr, len, -1);
		CORT_Task_Delay(CORT_TICK_PER_SEC);
	}
}
/*******************************************************************************
* Function Name  : MainTask
* Description    : 
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void MainTask(void *pd)
{
	t_u8 chr;
	t_device * pDev;
	t_uart_set uart;
	
	dev_uart_1_init();
	dev_hwinit_start();
	
	CORT_StateGo();
		
	pDev = CORT_DAI_Open(CORT_DAIT_UART, 1, -1);

	uart.baud_rate	= 115200;
	uart.stop_bits	= CORT_UART_STOPBIT_1;
	uart.Parity		= CORT_UART_PARITY_NONE;
	uart.byte_size	= CORT_UART_BYTESIZE_8;
	CORT_DAI_IOCtrl (pDev, eIOCtl_UART_SET, &uart, -1);

	CORT_Task_Create (UartSendTask, 0, STK_TASK_SEND, sizeof(STK_TASK_SEND), TASK_PRIO_SEND, "UartSendTask", m_null);
	while(1)
	{
		if (CORT_DAI_Read(pDev, &chr, 1, -1) == 1)
		{
			CORT_DAI_Write(pDev, &chr, 1, -1);
		}
	}
}
/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	dev_hwinit_init();
	
	CORT_Init();
	CORT_Task_Create (MainTask, 0, STK_TASK_MAIN, sizeof(STK_TASK_MAIN), TASK_PRIO_MAIN, "MainTask", m_null);
	CORT_Start();
}
