/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : main.c
* Author             : Jun Yao
* Email							 : nter@tom.com
* Version            : V1.0
* Date               : 07/22/2008
* Description        : Main program body
********************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "config.h"
#include "test_comm.c"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TASK_STK_SIZE	128
#define TASK_MAIN_PRIO	1
#define TASK_READ_PRIO	2
#define TASK_WRITE_PRIO	3

#define PACKSIZE	2
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
t_stk STK_TASK_MAIN[TASK_STK_SIZE];
t_stk STK_TASK_READ[TASK_STK_SIZE];
t_stk STK_TASK_WRITE[TASK_STK_SIZE];
t_pipe pipe;
t_u8 buffer[512];
t_tskid threadid1, threadid2;

/*******************************************************************************
* Function Name  : MainTask
* Description    : Main Task
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void WriteTask(void *pd)
{
	t_s32 num;
	t_u8 str[PACKSIZE+1]="h";
	
	while(1)
	{
		num = PACKSIZE;
#if TEST_DELETE
		switch(getchar())
		{
		case 'c':CORT_Pipe_Create (&pipe, buffer, sizeof(buffer));break;
		case 'd':CORT_Pipe_Delete (&pipe);   break;
		case 'w':CORT_Pipe_Write  (&pipe, str, &num, -1); break;
		}
#else
		if (CORT_Pipe_Write(&pipe, str, &num, -1) == CORT_OK)
		{
		}
#endif
	}
}
void ReadTask(void *pd)
{
	t_u8 str[PACKSIZE+1];
	t_s32 num;
	while(1)
	{
#if TEST_DELETE
		t_s8 ret;
		num = PACKSIZE;
		ret = CORT_Pipe_Read(&pipe, str, &num, 100);
		switch(ret)
		{
		case CORT_OK:
			CORT_Printf("Read ok\n");
			break;
		case CORT_ERR_TIMEOUT:
			CORT_Printf("Time Out\n");
			break;
		case CORT_ERR_INVALID_SUSPEND:
			CORT_Printf("Invalid Suspend\n");
			CORT_Task_Delay(100);
			break;
		case CORT_ERR_INVALID_HANDLE:
			CORT_Printf("Invalid handle\n");
			CORT_Task_Delay(100);
			break;
		case CORT_PIPE_DELETED:
			CORT_Printf("Queue deleted\n");
			break;
		}
#else
		num = PACKSIZE;
		if (CORT_Pipe_Read(&pipe, str, &num, -1) == CORT_OK)
		{
//			static t_s32 cnt = 0;
//			CORT_Printf("%d %c\n", cnt++, str[0]);
		}
#endif
	}
}
/*******************************************************************************
* Function Name  : MainTask
* Description    : Main Task
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void MainTask(void *pd)
{
	t_tasksta tasksta;
	t_u32 thread_tick1, thread_tick2,  total1, total2;
	
	dev_uart_1_init();
	dev_hwinit_start();
	
	CORT_StateGo();

	CORT_Printf_Init();

	CORT_Pipe_Create (&pipe, buffer, sizeof(buffer));
	CORT_Task_Create (ReadTask,  0, STK_TASK_READ, sizeof(STK_TASK_READ),  TASK_READ_PRIO, "ReadTask", &threadid1);
	CORT_Task_Create (WriteTask, 0, STK_TASK_WRITE,sizeof(STK_TASK_WRITE), TASK_WRITE_PRIO,"WriteTask",&threadid2);
	
	thread_tick1 = 0;
	thread_tick2 = 0;
	while(1)
	{
		CORT_Task_Delay(CORT_TICK_PER_SEC);
		
		total1 = total2 = 0;
		
		CORT_Task_GetState(threadid1, &tasksta);
		total1 = tasksta.ts_sched_cnt - thread_tick1; thread_tick1 = tasksta.ts_sched_cnt;
		
		CORT_Task_GetState(threadid2, &tasksta);
		total2 = tasksta.ts_sched_cnt - thread_tick2; thread_tick2 = tasksta.ts_sched_cnt;
		
		CORT_Printf("%d\n", total1 + total2);
	}
}

/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	dev_hwinit_init();
	
	CORT_Init();
	CORT_Task_Create (MainTask, 0, STK_TASK_MAIN, sizeof(STK_TASK_MAIN), TASK_MAIN_PRIO, "MainTask", m_null);
	CORT_Start();
}
