/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : main.c
* Author             : Jun Yao
* Email							 : nter@tom.com
* Version            : V1.0
* Date               : 07/22/2008
* Description        : Main program body
********************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "config.h"
#include "test_comm.c"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define TASK_STK_SIZE	128
#define TASK_MAIN_PRIO		1
#define TASK_OBTAIN_PRIO	2
#define TASK_RELEASE_PRIO	3


/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
t_stk STK_TASK_MAIN[TASK_STK_SIZE];
t_stk STK_TASK_OBTAIN[TASK_STK_SIZE];
t_stk STK_TASK_RELEASE[TASK_STK_SIZE];
t_sem sem;
t_tskid threadid1, threadid2;

/*******************************************************************************
* Function Name  : MainTask
* Description    : Main Task
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void ReleaseTask(void *pd)
{
	while(1)
	{
#if TEST_DELETE
		switch(getchar())
		{
		case 'c':CORT_Sem_Create(&sem, 0);break;
		case 'd':CORT_Sem_Delete(&sem);   break;
		case 'r':CORT_Sem_Release(&sem);  break;
		}
#else
		CORT_Sem_Release(&sem);
#endif
	}
}
void ObtainTask(void *pd)
{
	while(1)
	{
#if TEST_DELETE
		t_s8 ret = CORT_Sem_Obtain(&sem, 100);
		switch(ret)
		{
		case CORT_OK:
			CORT_Printf("Obtain ok\n");
			break;
		case CORT_ERR_TIMEOUT:
			CORT_Printf("Time Out\n");
			break;
		case CORT_ERR_INVALID_SUSPEND:
			CORT_Printf("Invalid Suspend\n");
			CORT_Task_Delay(100);
			break;
		case CORT_ERR_INVALID_HANDLE:
			CORT_Printf("Invalid handle\n");
			CORT_Task_Delay(100);
			break;
		case CORT_SEMAPHORE_DELETED:
			CORT_Printf("Semaphore deleted\n");
			break;
		}
#else
		CORT_Sem_Obtain(&sem, -1);
#endif
	}
}
/*******************************************************************************
* Function Name  : MainTask
* Description    : Main Task
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void MainTask(void *pd)
{
	t_tasksta tasksta;
	t_u32 thread_tick1, thread_tick2,  total1, total2;
	
	dev_uart_1_init();
	dev_hwinit_start();
	
	CORT_StateGo();

	CORT_Printf_Init();
		
	CORT_Sem_Create (&sem, 0);
	CORT_Task_Create (ObtainTask,  0,STK_TASK_OBTAIN, sizeof(STK_TASK_OBTAIN), TASK_OBTAIN_PRIO, "ObtainTask", &threadid1);
	CORT_Task_Create (ReleaseTask, 0,STK_TASK_RELEASE,sizeof(STK_TASK_RELEASE),TASK_RELEASE_PRIO,"ReleaseTask",&threadid2);

	thread_tick1 = 0;
	thread_tick2 = 0;
	while(1)
	{
		CORT_Task_Delay(CORT_TICK_PER_SEC);
		
		total1 = total2 = 0;
		
		CORT_Task_GetState(threadid1, &tasksta);
		total1 = tasksta.ts_sched_cnt - thread_tick1; thread_tick1 = tasksta.ts_sched_cnt;
		
		CORT_Task_GetState(threadid2, &tasksta);
		total2 = tasksta.ts_sched_cnt - thread_tick2; thread_tick2 = tasksta.ts_sched_cnt;
		
		CORT_Printf("%d\n", total1 + total2);
	}
}

/*******************************************************************************
* Function Name  : main
* Description    : Main program
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
int main(void)
{
	dev_hwinit_init();
	
	CORT_Init();
	CORT_Task_Create (MainTask, 0, STK_TASK_MAIN, sizeof(STK_TASK_MAIN), TASK_MAIN_PRIO, "MainTask", m_null);
	CORT_Start();
}
