/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

extern void CORT_Tick(void);

/*
**  Interrupt for System tick
*/
void dev_hwinit_irq(t_u16 idx)
{
	/* reload counter value */
	TCNT2 += 0xB2;
	/* System tick */
	CORT_Tick();
}

/*
**  Initialize Hardware
**   Target : M328P
**     Crystal: 8.0000Mhz
**   TIMER2 initialize - prescale:1024
**     WGM: Normal
**     desired value: 100Hz
**     actual value: 100.160Hz (0.2%)
*/
void dev_hwinit_init(void)
{
	/* PB5 Output */
	PORTB = 0x00;
	DDRB  = 0x20;
	PORTC = 0x00;
	DDRC  = 0x00;
	PORTD = 0x00;
	DDRD  = 0x00;
	
	/* Init Interrupt */
	MCUCR  = 0x00;
	TIMSK0 = 0x00; //timer interrupt sources
	TIMSK1 = 0x00; //timer interrupt sources
	TIMSK2 = 0x01; //timer interrupt sources
	PCMSK0 = 0x00; //pin change mask 0
	PCMSK1 = 0x00; //pin change mask 1
	PCMSK2 = 0x00; //pin change mask 2
	EIMSK  = 0x00;
	EICRA  = 0x00; //extended ext ints
	PCICR  = 0x00;

	/* Timer 2 */	
	/* Fmaster = 8MHz */
	TCCR2B = 0x00; //stop
	ASSR   = 0x00; //set async mode
	TCNT2  = 0xB2; //setup
	OCR2A  = 0x4E;
	OCR2B  = 0x00;
	TCCR2A = 0x00; 
	TCCR2B = 0x07; //start
	
	CORT_Interrupt_Register(9, dev_hwinit_irq);
}
/*
** Start timer
*/
void dev_hwinit_start(void)
{       
	__enable_interrupt();              /* Enable  interrupts */
}

