/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contents:
 *   CORT_InitCpu()        - 
 *   CORT_InitStk()        - 
 */

#include "cortos.h"
#include "cortos_common.h"
#include "config.h"

#define MT(x0,x1)	((x0)|(x1<<4))

const t_u8 CORT_PrioMapTbl[128] = {
    /*             0 1      2 3      4 5      6 7      8 9      A B      C D      E F*/
    /* 0x00 */  MT(0,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x10 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x20 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x30 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x40 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x50 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x60 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x70 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x80 */  MT(7,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x90 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xA0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xB0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xC0 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xD0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xE0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xF0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0)
};

t_u8 CORT_IrqIdx;

/*
**
*/
void   CORT_EnterCritical(void)
{
	__disable_interrupt();      		        /* Disable interrupts */
}

/*
**
*/
void   CORT_ExitCritical(void)
{
	__enable_interrupt();              /* Enable  interrupts */
}

/*
**
*/
t_u8  CORT_CountLeadZeros(t_prio val)
{
	t_u8 prio = CORT_PrioMapTbl[val>>1];
	if (val & 0x01)
	{
		prio >>= 4;
	}
	return prio & 0x0F;
}

/*
**
*/
t_stk *CORT_InitStk(void (*task)(void *p_arg), void *parg, t_stk *pstk, t_u32 size)
{
	t_u8 *stk_hw	= (t_u8*)pstk + (size - 1);
	t_u8 *stk_sw    = stk_hw - CORT_TASK_HARDSTK_SIZE;

	t_u16 addr_stkhw;
	t_u16 addr_task = (t_u16)task;
	t_u16 addr_parg = (t_u16)parg;

	//Put task start address on top of hardware stack
	
	//Software stack
	*(stk_hw--) = (t_u8)(addr_task&0xff);
	*(stk_hw--) = (t_u8)(addr_task>>8);

	*(--stk_sw) = 0x31;         //R31
	*(--stk_sw) = 0x30;         //R30
	*(--stk_sw) = 0x27;         //R27
	*(--stk_sw) = 0x26;         //R26
	*(--stk_sw) = 0x25;         //R25
	*(--stk_sw) = 0x24;         //R24
	*(--stk_sw) = 0x23;         //R23
	*(--stk_sw) = 0x22;         //R22
	*(--stk_sw) = 0x21;         //R21
	*(--stk_sw) = 0x20;         //R20
	*(--stk_sw) = 0x19;         //R19  
	*(--stk_sw) = 0x18;         //R18
	*(--stk_sw) = (t_u8)(addr_parg>>8);    //R17
	*(--stk_sw) = (t_u8)(addr_parg&0xff);  //R16
	*(--stk_sw) = 0x15;         //R15
	*(--stk_sw) = 0x14;         //R14
	*(--stk_sw) = 0x13;         //R13
	*(--stk_sw) = 0x12;         //R12
	*(--stk_sw) = 0x11;         //R11
	*(--stk_sw) = 0x10;         //R10
	*(--stk_sw) = 0x09;         //R9
	*(--stk_sw) = 0x08;         //R8
	*(--stk_sw) = 0x07;         //R7
	*(--stk_sw) = 0x06;         //R6
	*(--stk_sw) = 0x05;         //R5
	*(--stk_sw) = 0x04;         //R4
	*(--stk_sw) = 0x03;         //R3
	*(--stk_sw) = 0x02;         //R2
	*(--stk_sw) = 0x01;         //R1
	*(--stk_sw) = 0x00;         //R0
	*(--stk_sw) = 0x00;         //SREG Disable Interrupt

	addr_stkhw = (t_u16)stk_hw;
	*(--stk_sw) = (t_u8)(addr_stkhw>>8);
	*(--stk_sw) = (t_u8)(addr_stkhw&0xff);
	
	/* 34 byte */
	
	return (t_stk*)(stk_sw);
}

