/*
 * Copyright (C) Yao Jun 2012
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contents:
 *   CORT_IntEntry()       - 
 */

#include "cortos.h"
#include "config.h"


typedef void (*interrupt_handler_t)(t_u16);

void NonHandledInterrupt(t_u16 idx)
{
	while(1);
}

static interrupt_handler_t vec_tab[] = {
	{(interrupt_handler_t)NonHandledInterrupt}, //   1 0x000 RESET External Pin, Power-on Reset, Brown-out Reset and Watchdog System Reset
	{(interrupt_handler_t)NonHandledInterrupt}, //   2 0x001 INT0 External Interrupt Request 0
	{(interrupt_handler_t)NonHandledInterrupt}, //   3 0x002 INT1 External Interrupt Request 1
	{(interrupt_handler_t)NonHandledInterrupt}, //   4 0x003 PCINT0 Pin Change Interrupt Request 0
	{(interrupt_handler_t)NonHandledInterrupt}, //   5 0x004 PCINT1 Pin Change Interrupt Request 1
	{(interrupt_handler_t)NonHandledInterrupt}, //   6 0x005 PCINT2 Pin Change Interrupt Request 2
	{(interrupt_handler_t)NonHandledInterrupt}, //   7 0x006 WDT Watchdog Time-out Interrupt
	{(interrupt_handler_t)NonHandledInterrupt}, //   8 0x007 TIMER2 COMPA Timer/Counter2 Compare Match A
	{(interrupt_handler_t)NonHandledInterrupt},	//   9 0x008 TIMER2 COMPB Timer/Counter2 Compare Match B
	{(interrupt_handler_t)NonHandledInterrupt},	//  10 0x009 TIMER2 OVF Timer/Counter2 Overflow
	{(interrupt_handler_t)NonHandledInterrupt}, //  11 0x00A TIMER1 CAPT Timer/Counter1 Capture Event
	{(interrupt_handler_t)NonHandledInterrupt}, //  12 0x00B TIMER1 COMPA Timer/Counter1 Compare Match A
	{(interrupt_handler_t)NonHandledInterrupt}, //  13 0x00C TIMER1 COMPB Timer/C outner1 Compare Match B
	{(interrupt_handler_t)NonHandledInterrupt}, //  14 0x00D TIMER1 OVF Timer/Counter1 Overflow
	{(interrupt_handler_t)NonHandledInterrupt}, //  15 0x00E TIMER0 COMPA Timer/Counter0 Compare Match A
	{(interrupt_handler_t)NonHandledInterrupt}, //  16 0x00F TIMER0 COMPB Timer/C ounter0 Compare Match B
	{(interrupt_handler_t)NonHandledInterrupt}, //  17 0x010 TIMER0 OVF Timer/Counter0 Overflow
	{(interrupt_handler_t)NonHandledInterrupt}, //  18 0x011 SPI, STC SPI Serial Transfer Complete
	{(interrupt_handler_t)NonHandledInterrupt}, //  19 0x012 USART, RX USART Rx Complete
	{(interrupt_handler_t)NonHandledInterrupt}, //  20 0x013 USART, UDRE USART, Data Register Empty
	{(interrupt_handler_t)NonHandledInterrupt}, //  21 0x014 USART, TX USART, Tx Complete
	{(interrupt_handler_t)NonHandledInterrupt}, //  22 0x015 ADC ADC Conversion Complete
	{(interrupt_handler_t)NonHandledInterrupt}, //  23 0x016 EE READY EEPROM Ready
	{(interrupt_handler_t)NonHandledInterrupt}, //  24 0x017 ANALOG COMP Analog Comparator
	{(interrupt_handler_t)NonHandledInterrupt}, //  25 0x018 TWI 2-wire Serial Interface
	{(interrupt_handler_t)NonHandledInterrupt}, //  26 0x019 SPM READY Store Program Memory Ready
};

/*
**
*/
void CORT_IntRoutine(t_u8 id)
{
	if (id > 1 && id < CORT_ARRAYSIZE(vec_tab)){
		vec_tab[id](id);
	}
}

/*
**
*/
t_s8  CORT_Interrupt_Register(t_u16 idx,void (*task_entry)(t_u16))
{
	CORT_EnterCritical();
	if (idx > 1 && idx < CORT_ARRAYSIZE(vec_tab))
	{
		vec_tab[idx] = task_entry;
		CORT_ExitCritical();
		return CORT_OK;
	}
	CORT_ExitCritical();
	return CORT_ERROR;
}
