/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

extern void CORT_Tick(void);

/*
**
*/
static t_u16 dev_hwinit_rtmcnt(void)
{
	return 0;
}

/*
**  Interrupt for System tick
*/
static void dev_hwinit_sleep(void)
{
	
}

/*
**  Interrupt for System tick
*/
void dev_hwinit_tick(t_u16 idx)
{
	/* System tick */
	CORT_Tick();
}

void NVIC_Configuration(void)
{
	/* Set the Vector Table base location at 0x08000000 */ 
}

/*
**  Initialize Hardware
*/
void dev_hwinit_init(void)
{
	/*---------------------------------------------------------------------------------------------------------*/
	/* Init System Clock                                                                                       */
	/*---------------------------------------------------------------------------------------------------------*/
    /* Unlock protected registers */
    SYS_UnlockReg();

     /* Enable IRC22M clock */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_IRC22M_EN_Msk;

    /* Waiting for IRC22M clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_IRC22M_STB_Msk);

    /* Switch HCLK clock source to IRC22M */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_HCLK_IRC22M;

    /* Set PLL to Power-down mode and PLL_STB bit in CLKSTATUS register will be cleared by hardware.*/
    SYSCLK->PLLCON |= SYSCLK_PLLCON_PD_Msk;

    /* Enable external 12MHz XTAL, IRC10K */
    SYSCLK->PWRCON |= SYSCLK_PWRCON_XTL12M_EN_Msk | SYSCLK_PWRCON_OSC10K_EN_Msk;

    /* Enable PLL and Set PLL frequency */        
    SYSCLK->PLLCON = PLLCON_SETTING;

    /* Waiting for clock ready */
    SYS_WaitingForClockReady(SYSCLK_CLKSTATUS_PLL_STB_Msk | SYSCLK_CLKSTATUS_XTL12M_STB_Msk |SYSCLK_CLKSTATUS_IRC10K_STB_Msk);

    /* Switch HCLK clock source to PLL, STCLK to HCLK/2 */
    SYSCLK->CLKSEL0 = SYSCLK_CLKSEL0_STCLK_HCLK_DIV2 | SYSCLK_CLKSEL0_HCLK_PLL;

    /* Enable UART clock */        
    SYSCLK->APBCLK = 0;

    /* UART clock source */
    SYSCLK->CLKSEL1 = 0;

    /* Lock protected registers */
    SYS_LockReg();
	
    /* Update System Core Clock */
    /* User can use SystemCoreClockUpdate() to calculate PllClock, SystemCoreClock and CycylesPerUs automatically. */
    //SystemCoreClockUpdate(); 
    PllClock        = PLL_CLOCK;            // PLL
    SystemCoreClock = PLL_CLOCK / 1;        // HCLK
    CyclesPerUs     = PLL_CLOCK / 1000000;  // For SYS_SysTickDelay()
}
/*
** Start timer
*/
void dev_hwinit_start(void)
{
	t_hook hook = {0};
	
	hook.hk_sleep = dev_hwinit_sleep;
	hook.hk_rtmcnt = dev_hwinit_rtmcnt;
	
	CORT_Hook_RegCall(&hook);

	CORT_Interrupt_Register(15, dev_hwinit_tick);

	SysTick_Config(SystemCoreClock/1000);
}

