/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

static t_bool          led_0_init = m_false;
static struct t_device led_0_dev = {0};

/*
** Open Device
*/
t_s8 dev_led_0_open(void)
{
	if (!led_0_init)
	{
		led_0_init = m_true;

		_GPIO_SET_PIN_MODE(P3, 6, GPIO_PMD_OUTPUT);
	}
	
	return CORT_OK;
}

/*
** IO Control
*/
t_s8 dev_led_0_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_led_set * led = (t_led_set*)hParam;
	if (led && (nParam == eIOCtl_LED_SET))
	{
		if (led->mask & 0x0001)
		{
			if (led->onoff)
			{
				P36 = 0;		/* LED ON */
			}
			else
			{
				P36 = 1;		/* LED OFF */
			}
		}
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
t_s8 dev_led_0_standby(void)
{
	_GPIO_SET_PIN_MODE(P3, 6, GPIO_PMD_INPUT);
	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
t_s8 dev_led_0_resume(void)
{
	_GPIO_SET_PIN_MODE(P3, 6, GPIO_PMD_OUTPUT);
	
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
t_bool dev_led_0_allowstandby(void)
{
	return m_true;
}

/*
** Initialize led device
*/
void dev_led_0_init(void)
{	
	led_0_dev.dev_type		= CORT_DAIT_LED;
	led_0_dev.dev_idx		= 0;
	led_0_dev.open			= dev_led_0_open;;
	led_0_dev.ioctrl		= dev_led_0_ioctrl;
	led_0_dev.standby	 	= dev_led_0_standby;
	led_0_dev.resume	 	= dev_led_0_resume;
	led_0_dev.allow_standby = dev_led_0_allowstandby;
	
	CORT_DAI_RegDev(&led_0_dev);
}

