/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"


static t_u8     uart_1_t_buf[64];
static t_u8     uart_1_r_buf[64];
static t_pipe   uart_1_t_pipe;
static t_pipe   uart_1_r_pipe;
static t_device uart_1_dev = {0};
static t_bool   uart_1_init = m_false;
static t_bool   uart_1_standby = m_true;

/*
**  Interrupt Service
*/
static void dev_uart_1_irq(t_u16 idx)
{
	t_u8  u8Char;
	t_s32 u32Num;
    t_u32 u32IntSts= UART0->ISR;

    if(u32IntSts & UART_ISR_RDA_INT_Msk)
    {      
        /* Get all the input characters */
        while(_UART_IS_RX_READY(UART0)) 
        {
            /* Get the character from UART Buffer */
            _UART_RECEIVEBYTE(UART0, u8Char);           /* Rx trigger level is 1 byte*/    
			
			/* Write char to queue */
			u32Num = 1;
			CORT_Pipe_Write(&uart_1_r_pipe, &u8Char, &u32Num, 0);
        }
    }

    if(u32IntSts & UART_ISR_THRE_INT_Msk)
    {   
		/* Write one byte to the transmit data register */   
		u32Num = 1;
		if (CORT_Pipe_Read(&uart_1_t_pipe, &u8Char, &u32Num, -1) == CORT_OK)
		{
			_UART_SENDBYTE(UART0,u8Char);
		}
		else
		{
			 _UART_DISABLE_INT(UART0, UART_IER_THRE_IEN_Msk);
		}
    }
}

/*
** Open Device
*/
static t_s8 dev_uart_1_open(void)
{
	if (!uart_1_init)
	{
		uart_1_init = m_true;
		uart_1_standby = m_false;
		
	    /* Unlock protected registers */
	    SYS_UnlockReg();

	    /* Set P3 multi-function pins for UART0 RXD and TXD  */
	    SYS->P3_MFP = SYS_MFP_P30_RXD0 | SYS_MFP_P31_TXD0;

		/* Enable peripheral clock */        
	    SYSCLK->APBCLK |= SYSCLK_APBCLK_UART0_EN_Msk;

	    /* Peripheral clock source */
	    SYSCLK->CLKSEL1 |= SYSCLK_CLKSEL1_UART_XTAL;

		/* Reset UART */
		SYS->IPRSTC2 |=  SYS_IPRSTC2_UART0_RST_Msk;
		SYS->IPRSTC2 &= ~SYS_IPRSTC2_UART0_RST_Msk;

	    /* Lock protected registers */
	    SYS_LockReg();

		/* Configure UART0 and set UART0 Baudrate */
		UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(__XTAL,115200);
	    _UART_SET_DATA_FORMAT(UART0, UART_WORD_LEN_8 | UART_PARITY_NONE | UART_STOP_BIT_1);
		
	    /* Enable Interrupt and install the call back function */
	    _UART_ENABLE_INT(UART0, UART_IER_RDA_IEN_Msk);
	    NVIC_EnableIRQ(UART0_IRQn);
	}
	return CORT_OK;
}

/*
** Read for uart
*/
static t_s32 dev_uart_1_read(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s8 ret = CORT_Pipe_Read(&uart_1_r_pipe, pBuf, &nLen, tick);

	if (ret == CORT_OK ||
		ret == CORT_ERR_TIMEOUT)
	{
		return nLen;
	}
	return -1;
}

/*
** Write to uart
*/
static t_s32 dev_uart_1_write(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s8 ret = CORT_Pipe_Write(&uart_1_t_pipe, pBuf, &nLen, tick);

	if (ret == CORT_OK ||
		ret == CORT_ERR_TIMEOUT)
	{
		if (nLen)
		{
			if (!((UART0)->IER & UART_IER_THRE_IEN_Msk))
			{
				/* Enable the USARTx Transmit interrupt */
				 _UART_ENABLE_INT(UART0, UART_IER_THRE_IEN_Msk);
			}
		}
		return nLen;
	}
	return -1;
}

/*
** IO Control
*/
static t_s8 dev_uart_1_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_u32 format = 0;
	t_uart_set * uart = (t_uart_set*)hParam;
	
	if (uart && (nParam == eIOCtl_UART_SET))
	{
		
		
		switch(uart->stop_bits)
		{
		case CORT_UART_STOPBIT_1:
			format |= UART_STOP_BIT_1;
			break;
		case CORT_UART_STOPBIT_1_5:
			format |= UART_STOP_BIT_1_5;
			break;
		case CORT_UART_STOPBIT_2:
			format |= UART_STOP_BIT_2;
			break;
		}
		switch(uart->Parity)
		{
		case CORT_UART_PARITY_ODD:
			format |= UART_PARITY_ODD;
			break;
		case CORT_UART_PARITY_EVEN:
			format |= UART_PARITY_EVEN;
			break;
		default:
			format |= UART_PARITY_NONE;
			break;
		}
		switch(uart->byte_size)
		{
		case CORT_UART_BYTESIZE_5:
			format |= UART_WORD_LEN_5;
			break;
		case CORT_UART_BYTESIZE_6:
			format |= UART_WORD_LEN_6;
			break;
		case CORT_UART_BYTESIZE_7:
			format |= UART_WORD_LEN_7;
			break;
		case CORT_UART_BYTESIZE_8:
			format |= UART_WORD_LEN_8;
			break;
		}

		UART0->BAUD = UART_BAUD_MODE2 | UART_BAUD_DIV_MODE2(__XTAL, uart->baud_rate);
		_UART_SET_DATA_FORMAT(UART0, format);
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
static t_s8 dev_uart_1_standby(void)
{
	uart_1_standby = m_true;
	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
static t_s8 dev_uart_1_resume(void)
{
	uart_1_standby = m_false;
	
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
static t_bool dev_uart_1_allow_standby(void)
{
	return uart_1_standby;
}

/*
** Initialize led device
*/
void dev_uart_1_init(void)
{	
	CORT_Interrupt_Register(28, dev_uart_1_irq);

	CORT_Pipe_Create(&uart_1_t_pipe, uart_1_t_buf, sizeof(uart_1_t_buf));
	CORT_Pipe_Create(&uart_1_r_pipe, uart_1_r_buf, sizeof(uart_1_r_buf));
	
	uart_1_dev.dev_type		= CORT_DAIT_UART;
	uart_1_dev.dev_idx		= 1;
	uart_1_dev.open			= dev_uart_1_open;
	uart_1_dev.read			= dev_uart_1_read;
	uart_1_dev.write		= dev_uart_1_write;
	uart_1_dev.ioctrl	 	= dev_uart_1_ioctrl;
	uart_1_dev.standby	 	= dev_uart_1_standby;
	uart_1_dev.resume	 	= dev_uart_1_resume;
	uart_1_dev.allow_standby= dev_uart_1_allow_standby;
	
	CORT_DAI_RegDev(&uart_1_dev);
}


