/**************************************************************************//**
 * @file     WWDT.h
 * @version  V2.03
 * $Revision: 2 $
 * $Date: 13/06/28 9:47a $
 * @brief    M051 Series Window Watchdog Timer Driver Header File
 *
 * @note
 * Copyright (C) 2011 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#ifndef __WWDT_H__
#define __WWDT_H__

#include "M051Series.h"


/** @addtogroup M051_FUNC M051 Function Interface
  * @{
  */

/** @addtogroup WWDT_FUNC WWDT Device Function Interface
  * @{
  */


/*---------------------------------------------------------------------------------------------------------*/
/* WWDTCR Constants Definitions                                                                            */                                                                            
/*---------------------------------------------------------------------------------------------------------*/
#define WWDT_WWDTCR_WINCMP(x)       ((x) << WWDT_WWDTCR_WINCMP_Pos)   /*!< WWDTCR setting for adjust the valid reload window. 0 <= x <= 0x3F */

#define WWDT_WWDTCR_PERIODSEL_1     (0 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 1 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_2     (1 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 2 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_4     (2 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 4 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_8     (3 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 8 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_16    (4 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 16 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_32    (5 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 32 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_64    (6 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 64 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_128   (7 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 128 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_192   (8 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 192 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_256   (9 << WWDT_WWDTCR_PERIODSEL_Pos)  /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 256 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_384   (10 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 384 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_512   (11 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 512 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_768   (12 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 768 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_1024  (13 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 1024 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_1536  (14 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 1536 * (64*WWDT_CLK) */
#define WWDT_WWDTCR_PERIODSEL_2048  (15 << WWDT_WWDTCR_PERIODSEL_Pos) /*!< WWDTCR setting to select the pre-scale for the WWDT counter period. Max timeout period is 2048 * (64*WWDT_CLK) */


/*---------------------------------------------------------------------------------------------------------*/
/* WWDT Macro Definitions                                                                                  */                                                                            
/*---------------------------------------------------------------------------------------------------------*/
/**
 * @details     Enable WWDT to start counting
 *
 * @param[in]   wincmp      The WWDT valid reload window. 0 <= wincmp <= 0x3F.
 * @param[in]   periodsel   The pre-scale for the WWDT counter period. It could be \n
 *                              WWDT_WWDTCR_PERIODSEL_1, WWDT_WWDTCR_PERIODSEL_2, WWDT_WWDTCR_PERIODSEL_4, WWDT_WWDTCR_PERIODSEL_8, \n
 *                              WWDT_WWDTCR_PERIODSEL_16, WWDT_WWDTCR_PERIODSEL_32, WWDT_WWDTCR_PERIODSEL_64, WWDT_WWDTCR_PERIODSEL_128, \n
 *                              WWDT_WWDTCR_PERIODSEL_192, WWDT_WWDTCR_PERIODSEL_256, WWDT_WWDTCR_PERIODSEL_384, WWDT_WWDTCR_PERIODSEL_512, \n
 *                              WWDT_WWDTCR_PERIODSEL_768, WWDT_WWDTCR_PERIODSEL_1024, WWDT_WWDTCR_PERIODSEL_1536, WWDT_WWDTCR_PERIODSEL_2048.
 *
 * @return      None
 *
 * @details     To start WWDT counting and WWDT interrupt function is disabled.
 */
#define _WWDT_ENABLE(wincmp, periodsel)         (WWDT->WWDTCR = (WWDT_WWDTCR_WINCMP(wincmp) | (periodsel) | WWDT_WWDTCR_WWDTEN_Msk))

/**
 * @details     Enable WWDT to start counting with WWDT interrupt enable
 *
 * @param[in]   wincmp      The WWDT valid reload window. 0 <= wincmp <= 0x3F.
 * @param[in]   periodsel   The pre-scale for the WWDT counter period. It could be \n
 *                              WWDT_WWDTCR_PERIODSEL_1, WWDT_WWDTCR_PERIODSEL_2, WWDT_WWDTCR_PERIODSEL_4, WWDT_WWDTCR_PERIODSEL_8, \n
 *                              WWDT_WWDTCR_PERIODSEL_16, WWDT_WWDTCR_PERIODSEL_32, WWDT_WWDTCR_PERIODSEL_64, WWDT_WWDTCR_PERIODSEL_128, \n
 *                              WWDT_WWDTCR_PERIODSEL_192, WWDT_WWDTCR_PERIODSEL_256, WWDT_WWDTCR_PERIODSEL_384, WWDT_WWDTCR_PERIODSEL_512, \n
 *                              WWDT_WWDTCR_PERIODSEL_768, WWDT_WWDTCR_PERIODSEL_1024, WWDT_WWDTCR_PERIODSEL_1536, WWDT_WWDTCR_PERIODSEL_2048.
 *
 * @return      None
 *
 * @details     To start WWDT counting and enable WWDT interrupt function.
 */
#define _WWDT_ENABLE_IE(wincmp, periodsel)      (WWDT->WWDTCR = (WWDT_WWDTCR_WINCMP(wincmp) | (periodsel) | WWDT_WWDTCR_WWDTIE_Msk | WWDT_WWDTCR_WWDTEN_Msk))
 
/**
 * @details     To reload the WWDT counter value to 0x3F
 */
#define _WWDT_RELOAD_COUNTER()                  (WWDT->WWDTRLD = 0x00005AA5)
 
/**
 * @details     To get the counter value of WWDT
 */
#define _WWDT_GET_COUNTER()                     (WWDT->WWDTCVR)

/**
 * @details     Get WWDT Compare Match Interrupt Flag
 */
#define _WWDT_GET_CMP_INT_FLAG()                ((((WWDT->WWDTSR) & WWDT_WWDTSR_WWDTIF_Msk) == WWDT_WWDTSR_WWDTIF_Msk)? 1:0)

/**
 * @details     Clear WWDT Compare Match Interrupt Flag to 0
 */
#define _WWDT_CLEAR_CMP_INT_FLAG()              (WWDT->WWDTSR = WWDT_WWDTSR_WWDTIF_Msk)

/**
 * @details     Get WWDT Timeout Reset Flag
 */
#define _WWDT_GET_RESET_FLAG()                  ((((WWDT->WWDTSR) & WWDT_WWDTSR_WWDTRF_Msk) == WWDT_WWDTSR_WWDTRF_Msk)? 1:0)

/**
 * @details     Clear WWDT Timeout Reset Flag to 0
 */
#define _WWDT_CLEAR_RESET_FLAG()                (WWDT->WWDTSR = WWDT_WWDTSR_WWDTRF_Msk)


/**
  * @} End of WWDT Device Function Interface
  */ 

/**
  * @} End of M051 Function Interface
  */ 


#endif  // __WWDT_H__ 
