/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef   __CORTOS_CONF_H
#define   __CORTOS_CONF_H


#define CORT_KERNEL_LITE		1
#define CORT_KERNEL_FULL		0

/* ------------------------------------------------------ MISCELLANEOUS ---------------------------------   */
#define CORT_LOWEST_PRIO            4       /* Defines the lowest priority that can be assigned ... (2-31)  */
#define CORT_TICK_PER_SEC        1000       /* Set the number of ticks in one second                        */
#define CORT_STAT_PRE_SEC          10       /* Set the number of  calculate state in one second             */
#define CORT_CHECK_ALIVE	    	0		/* Enable (1) or Disable (0) check Content of struct            */
#define CORT_CHECK_PARAM            1       /* Enable (1) or Disable (0) argument checking                  */
/* ------------------------------------------------------ Full Version ------------------------------------ */
#define CORT_MAX_TASK              62       /* Max task                                                     */
/* ------------------------------------------------Driver Application Interface --------------------------- */
#define CORT_DAI_EN                 1       /* Enable (1) or Disable (0)                                    */
#define CORT_DAI_CLOSE              0       /*  Include code for CORT_DAI_Close()                           */
#define CORT_DAI_READ               1       /*  Include code for CORT_DAI_Read()                            */
#define CORT_DAI_WRITE              1       /*  Include code for CORT_DAI_Write()                           */
#define CORT_DAI_IOCTRL             1       /*  Include code for CORT_DAI_IOCtrl()                          */
#define CORT_DAI_STANDBY            1       /*  Include code for CORT_DAI_Standby()                         */
                                            /*                   CORT_DAI_Resume()                          */
                                            /*                   CORT_DAI_IsAllowStandby()                  */
#define CORT_DAI_READ_PROTECT_EN	1       /*                                                              */
#define CORT_DAI_WRITE_PROTECT_EN	1       /*                                                              */
#define CORT_DAI_IOCTRL_PROTECT_EN	1       /*                                                              */
/* ------------------------------------------------------ SYSTEM TASK STACK SIZE -------------------------- */
#define CORT_IDLE_STK_SIZE         64       /* Idle task stack size                                         */
#define CORT_HIGH_STK_SIZE        128       /* High task stack size                                         */
#define CORT_INTE_STK_SIZE          0       /* stack size for Interrupt Routine                             */
/* ------------------------------------------------------ OS CONTROL -------------------------------------- */
#define CORT_SCHEDLOCK_EN           1       /*  Include code for CORT_SchedLock() and CORT_SchedUnLock()    */
/* ------------------------------------------------------ OS Hook ----------------------------------------- */
#define CORT_HOOK_EN				1		/* Enable (1) or Disable (0) hook                               */
#define CORT_HOOK_TASK_CREATE_EN    0       /*                                                              */
#define CORT_HOOK_TASK_DELETE_EN    0       /*                                                              */
#define CORT_HOOK_TASK_STATE_EN     0       /*                                                              */
#define CORT_HOOK_TASK_SWITCH_EN    0       /*                                                              */
#define CORT_HOOK_TASK_ERROR_EN     0       /*                                                              */
#define CORT_HOOK_TICK_EN           0       /*                                                              */
#define CORT_HOOK_SLEEP_EN          0       /*                                                              */
/* ------------------------------------------------------ TASK MANAGEMENT --------------------------------- */
#define CORT_TASK_DEL_EN            0       /*  Include code for CORT_Task_Delete()                         */
#define CORT_TASK_SUSPEND_EN        0       /*  Include code for CORT_Task_Suspend() and CORT_Task_Resume() */
#define CORT_TASK_GETID_EN          0       /*  Include code for CORT_Task_GetID()                          */
#define CORT_TASK_SCHEDCNT_EN       1       /*  Include code for CORT_Task_GetSchedCount()                  */
#define CORT_TASK_GETSTAT_EN        1       /*  Include code for CORT_Task_GetState()                       */
#define CORT_TASK_GETLIST_EN        0       /*  Include code for CORT_Task_GetNum() and CORT_Task_GetNext() */
#define CORT_TASK_RUNTIME_EN        0       /*                                                              */
#define CORT_TASK_NAME_EN           0       /*  Enable task names                                           */
#define CORT_TASK_CHECK_STK_EN      0       /*  Enable Check task stacks                                    */
/* ------------------------------------------------------ SEMAPHORE --------------------------------------- */
#define CORT_SEM_EN                 1       /* Enable (1) or Disable (0) code generation for SEMAPHORE      */
#define CORT_SEM_DEL_EN             1       /*  Include code for CORT_Sem_Delete()                          */
/* ------------------------------------------------------ MESSAGE QUEUES ---t------------------------------ */
#define CORT_QUEUE_EN               1       /* Enable (1) or Disable (0) code generation for MESSAGE QUEUES */
#define CORT_QUEUE_DEL_EN           1       /*  Include code for CORT_Pipe_Delete()                         */
/* ------------------------------------------------------ MESSAGE PIPE ------------------------------------ */
#define CORT_PIPE_EN                1       /* Enable (1) or Disable (0) code generation for MESSAGE PIPE   */
#define CORT_PIPE_DEL_EN            1       /*  Include code for CORT_Pipe_Delete()                         */
/* ------------------------------------------------------ MAILBOXES --------------------------------------- */
#define CORT_MAIL_EN                1       /* Enable (1) or Disable (0) code generation for MAILBOXES      */
#define CORT_MAIL_DEL_EN            1       /*  Include code for CORT_Mailbox_Delete()                      */
#define CORT_MAIL_QUEUE_SIZE        4       /*                                                              */
/* ------------------------------------------------------ EVENT FLAGS ------------------------------------- */
#define CORT_FLAG_EN                1       /* Enable (1) or Disable (0) code generation for EVENT FLAGS    */
#define CORT_FLAG_DEL_EN            1       /*  Include code for CORT_Flag_Delete()                         */
/* ------------------------------------------------------ TIME MANAGEMENT --------------------------------- */
#define CORT_TIMER_EN               1       /* Enable (1) or Disable (0) code generation for TIMER          */
#define CORT_TIMER_DEL_EN           1       /*  Include code for CORT_Timer_Delete()                        */
#define CORT_TIMER_RESET_EN         1       /*  Include code for CORT_Timer_Reset()                         */
#define CORT_TIMER_STARTSTOP_EN     1       /*  Include code for CORT_Timer_Start() and CORT_Timer_Stop()   */
#define CORT_TIMER_SET_EN           1       /*  Include code for CORT_Timer_Set()                           */
/* -------------------------------------------------------------------------------------------------------- */

#endif	//#ifndef   __CORTOS_CONF_H

