/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "cortos.h"
#include "cortos_common.h"

#define MT(x0,x1)	((x0)|(x1<<4))

const t_u8 CORT_PrioMapTbl[128] = {
	/*             0 1      2 3      4 5      6 7      8 9      A B      C D      E F*/
	/* 0x00 */  MT(0,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x10 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x20 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x30 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x40 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x50 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x60 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x70 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x80 */  MT(7,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x90 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xA0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xB0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xC0 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xD0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xE0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xF0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0)
};

/*
**
*/
t_u8  CORT_CountLeadZeros(t_prio val)
{
	t_u8 prio = CORT_PrioMapTbl[val>>1];
	if (val & 0x01)
	{
		prio >>= 4;
	}
	return prio & 0x0F;
}
/*
**
*/
t_stk *CORT_InitStk(t_void (*task)(t_void *p_arg), t_void *parg, t_stk *pstk, t_size size)
{
    t_u32 *stk = &pstk[size/4];

	*(--stk)  = (t_u32)0x21000000L;            /* xPSR */
	*(--stk)  = (t_u32)task;                   /* Entry Point */
	*(--stk)  = (t_u32)0xFFFFFFFEL;            /* R14 (LR) */
	*(--stk)  = (t_u32)0x12121212L;            /* R12 */
	*(--stk)  = (t_u32)0x03030303L;            /* R3 */
	*(--stk)  = (t_u32)0x02020202L;            /* R2 */
	*(--stk)  = (t_u32)0x01010101L;            /* R1 */
	*(--stk)  = (t_u32)parg;                   /* R0 : argument */
	
	*(--stk)  = (t_u32)0x07070707L;            /* R7 */
	*(--stk)  = (t_u32)0x06060606L;            /* R6 */
	*(--stk)  = (t_u32)0x05050505L;            /* R5 */
	*(--stk)  = (t_u32)0x04040404L;            /* R4 */
	*(--stk)  = (t_u32)0x11111111L;            /* R11 */
	*(--stk)  = (t_u32)0x10101010L;            /* R10 */
	*(--stk)  = (t_u32)0x09090909L;            /* R9 */
	*(--stk)  = (t_u32)0x08080808L;            /* R8 */
    return (stk);
}

