;******************** (C) COPYRIGHT 2010 STMicroelectronics ********************
;* File Name          : startup_stm32f10x_hd.s
;* Author             : MCD Application Team
;* Version            : V3.3.0
;* Date               : 04/16/2010
;* Description        : STM32F10x High Density Devices vector table for RVMDK 
;*                      toolchain. 
;*                      This module performs:
;*                      - Set the initial SP
;*                      - Set the initial PC == Reset_Handler
;*                      - Set the vector table entries with the exceptions ISR address
;*                      - Configure the clock system and also configure the external 
;*                        SRAM mounted on STM3210E-EVAL board to be used as data 
;*                        memory (optional, to be enabled by user)
;*                      - Branches to __main in the C library (which eventually
;*                        calls main()).
;*                      After Reset the CortexM3 processor is in Thread mode,
;*                      priority is Privileged, and the Stack is set to Main.
;* <<< Use Configuration Wizard in Context Menu >>>   
;*******************************************************************************
; THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
; WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
; AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
; INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
; CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
; INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
;*******************************************************************************

; Amount of memory (in bytes) allocated for Stack
; Tailor this value to your application needs
; <h> Stack Configuration
;   <o> Stack Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Stack_Size      EQU     0x00000400

                AREA    STACK, NOINIT, READWRITE, ALIGN=3
Stack_Mem       SPACE   Stack_Size
__initial_sp
                                                  
; <h> Heap Configuration
;   <o>  Heap Size (in Bytes) <0x0-0xFFFFFFFF:8>
; </h>

Heap_Size       EQU     0x00000200

                AREA    HEAP, NOINIT, READWRITE, ALIGN=3
__heap_base
Heap_Mem        SPACE   Heap_Size
__heap_limit

                PRESERVE8
                THUMB


; Vector Table Mapped to Address 0 at Reset
                AREA    RESET, DATA, READONLY
                EXTERN	CORT_GlobalInt
                EXTERN  CORT_PendSV
                EXTERN  HardFaultHandler
                EXPORT  __Vectors
                EXPORT  __Vectors_End
                EXPORT  __Vectors_Size

__Vectors       DCD     __initial_sp          ; 0 Top of Stack
                DCD     Reset_Handler         ; 1 Reset Handler
                DCD     CORT_GlobalInt        ; 2 NMI Handler
                DCD     HardFaultHandler      ; 3 Hard Fault Handler
                DCD     CORT_GlobalInt        ; 4 MPU Fault Handler
                DCD     CORT_GlobalInt        ; 5 Bus Fault Handler
                DCD     CORT_GlobalInt        ; 6 Usage Fault Handler
                DCD     0                     ; 7 Reserved
                DCD     0                     ; 8 Reserved
                DCD     0                     ; 9 Reserved
                DCD     0                     ;10 Reserved
                DCD     CORT_GlobalInt        ;11 SVCall Handler
                DCD     CORT_GlobalInt        ;12 Debug Monitor Handler
                DCD     0                     ;13 Reserved
                DCD     CORT_PendSV           ;14 PendSV Handler
                DCD     CORT_GlobalInt        ;15 SysTick Handler
                DCD     CORT_GlobalInt        ;16 Window Watchdog
                DCD     CORT_GlobalInt        ;17 PVD through EXTI Line detect
                DCD     CORT_GlobalInt        ;18 Tamper
                DCD     CORT_GlobalInt        ;19 RTC
                DCD     CORT_GlobalInt        ;20 Flash
                DCD     CORT_GlobalInt        ;21 RCC
                DCD     CORT_GlobalInt        ;22 EXTI Line 0
                DCD     CORT_GlobalInt        ;23 EXTI Line 1
                DCD     CORT_GlobalInt        ;24 EXTI Line 2
                DCD     CORT_GlobalInt        ;25 EXTI Line 3
                DCD     CORT_GlobalInt        ;26 EXTI Line 4
                DCD     CORT_GlobalInt        ;27 DMA1 Channel 1
                DCD     CORT_GlobalInt        ;28 DMA1 Channel 2
                DCD     CORT_GlobalInt        ;29 DMA1 Channel 3
                DCD     CORT_GlobalInt        ;30 DMA1 Channel 4
                DCD     CORT_GlobalInt        ;31 DMA1 Channel 5
                DCD     CORT_GlobalInt        ;32 DMA1 Channel 6
                DCD     CORT_GlobalInt        ;33 DMA1 Channel 7
                DCD     CORT_GlobalInt        ;34 ADC1 & ADC2
                DCD     CORT_GlobalInt        ;35 USB High Priority or CAN1 TX
                DCD     CORT_GlobalInt        ;36 USB Low  Priority or CAN1 RX0
                DCD     CORT_GlobalInt        ;37 CAN1 RX1
                DCD     CORT_GlobalInt        ;38 CAN1 SCE
                DCD     CORT_GlobalInt        ;39 EXTI Line 9..5
                DCD     CORT_GlobalInt        ;40 TIM1 Break
                DCD     CORT_GlobalInt        ;41 TIM1 Update
                DCD     CORT_GlobalInt        ;42 TIM1 Trigger and Commutation
                DCD     CORT_GlobalInt        ;43 TIM1 Capture Compare
                DCD     CORT_GlobalInt        ;44 TIM2
                DCD     CORT_GlobalInt        ;45 TIM3
                DCD     CORT_GlobalInt        ;46 TIM4
                DCD     CORT_GlobalInt        ;47 I2C1 Event
                DCD     CORT_GlobalInt        ;48 I2C1 Error
                DCD     CORT_GlobalInt        ;49 I2C2 Event
                DCD     CORT_GlobalInt        ;50 I2C2 Error
                DCD     CORT_GlobalInt        ;51 SPI1
                DCD     CORT_GlobalInt        ;52 SPI2
                DCD     CORT_GlobalInt        ;53 USART1
                DCD     CORT_GlobalInt        ;54 USART2
                DCD     CORT_GlobalInt        ;55 USART3
                DCD     CORT_GlobalInt        ;56 EXTI Line 15..10
                DCD     CORT_GlobalInt        ;57 RTC Alarm through EXTI Line
                DCD     CORT_GlobalInt        ;58 USB Wakeup from suspend
                DCD     CORT_GlobalInt        ;59 TIM8 Break
                DCD     CORT_GlobalInt        ;60 TIM8 Update
                DCD     CORT_GlobalInt        ;61 TIM8 Trigger and Commutation
                DCD     CORT_GlobalInt        ;62 TIM8 Capture Compare
                DCD     CORT_GlobalInt        ;63 ADC3
                DCD     CORT_GlobalInt        ;64 FSMC
                DCD     CORT_GlobalInt        ;65 SDIO
                DCD     CORT_GlobalInt        ;66 TIM5
                DCD     CORT_GlobalInt        ;67 SPI3
                DCD     CORT_GlobalInt        ;68 UART4
                DCD     CORT_GlobalInt        ;69 UART5
                DCD     CORT_GlobalInt        ;70 TIM6
                DCD     CORT_GlobalInt        ;71 TIM7
                DCD     CORT_GlobalInt        ;72 DMA2 Channel1
                DCD     CORT_GlobalInt        ;73 DMA2 Channel2
                DCD     CORT_GlobalInt        ;74 DMA2 Channel3
                DCD     CORT_GlobalInt        ;75 DMA2 Channel4 & Channel5         
__Vectors_End

__Vectors_Size  EQU  __Vectors_End - __Vectors

                AREA    |.text|, CODE, READONLY
                
; Reset handler
Reset_Handler   PROC
                EXPORT  Reset_Handler             [WEAK]
                IMPORT  __main
                IMPORT  SystemInit
                LDR     R0, =SystemInit
                BLX     R0               
                LDR     R0, =__main
                BX      R0
                ENDP

                ALIGN

;*******************************************************************************
; User Stack and Heap initialization
;*******************************************************************************
                 IF      :DEF:__MICROLIB
                
                 EXPORT  __initial_sp
                 EXPORT  __heap_base
                 EXPORT  __heap_limit
                
                 ELSE
                
                 IMPORT  __use_two_region_memory
                 EXPORT  __user_initial_stackheap
                 
__user_initial_stackheap

                 LDR     R0, =  Heap_Mem
                 LDR     R1, =(Stack_Mem + Stack_Size)
                 LDR     R2, = (Heap_Mem +  Heap_Size)
                 LDR     R3, = Stack_Mem
                 BX      LR

                 ALIGN

                 ENDIF

                 END

;******************* (C) COPYRIGHT 2010 STMicroelectronics *****END OF FILE*****
