/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

extern void CORT_Tick(void);

/*
**
*/
static t_u16 dev_hwinit_rtmcnt(void)
{
	return TIM1->CNT;
}

/*
**  Interrupt for System tick
*/
static void dev_hwinit_sleep(void)
{
	PWR->CR &= ~6;
	__WFI();
}

/*
**  Interrupt for System tick
*/
void dev_hwinit_tick(t_u16 idx)
{
	/* System tick */
	CORT_Tick();
}

void NVIC_Configuration(void)
{
	/* Set the Vector Table base location at 0x08000000 */ 
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
}

/*
**
*/
void dev_hwinit_timer(void)
{
	RCC_ClocksTypeDef clk;
	TIM_TimeBaseInitTypeDef  TIM_TimeBaseStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);

	RCC_GetClocksFreq(&clk);
	
	/* Time base configuration */
	TIM_TimeBaseStructure.TIM_Period = 0xffff;
	TIM_TimeBaseStructure.TIM_Prescaler = (clk.PCLK2_Frequency/1000000)-1;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;    
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up; 
	TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

	/* TIM enable counter */
	TIM_Cmd(TIM1, ENABLE);
}

/*
**  Initialize Hardware
*/
void dev_hwinit_init(void)
{
//FLASH_DataCacheCmd(ENABLE);
//FLASH_InstructionCacheCmd(ENABLE);	

	CORT_Interrupt_Register(15, dev_hwinit_tick);

	/* System Clocks Configuration */
	SystemInit();

	/* Enable Clock */
	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);

	/* NVIC configuration */
	NVIC_Configuration();
}
/*
** Start timer
*/
void dev_hwinit_start(void)
{
	t_hook hook = {0};
	RCC_ClocksTypeDef RCC_Clocks;

	hook.hk_sleep = dev_hwinit_sleep;
	hook.hk_rtmcnt = dev_hwinit_rtmcnt;
	
	CORT_Hook_RegCall(&hook);

	RCC_GetClocksFreq(&RCC_Clocks);
	SysTick_Config(RCC_Clocks.HCLK_Frequency / CORT_TICK_PER_SEC);
  
	dev_hwinit_timer();
}

