/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

static t_bool          led_0_init = m_false;
static struct t_device led_0_dev = {0};

/*
** Open Device
*/
t_s8 dev_led_0_open(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	if (!led_0_init)
	{
		//PD12~PD15
		led_0_init = m_true;

		RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD, ENABLE);

		/* Configure IO in output pushpull mode */
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		GPIO_Init(GPIOD, &GPIO_InitStructure);
	}
	return CORT_OK;
}

/*
** IO Control
*/
t_s8 dev_led_0_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_led_set * led = (t_led_set*)hParam;
	if (led && (nParam == eIOCtl_LED_SET))
	{
		if (led->mask & 0x0001)
		{
			if (led->onoff)
			{
				GPIO_SetBits(GPIOD, GPIO_Pin_12);	/* LED ON */
			}
			else
			{
				GPIO_ResetBits(GPIOD, GPIO_Pin_12);/* LED OFF */
			}
		}
		if (led->mask & 0x0002)
		{
			if (led->onoff)
			{
				GPIO_SetBits(GPIOD, GPIO_Pin_13);	/* LED ON */
			}
			else
			{
				GPIO_ResetBits(GPIOD, GPIO_Pin_13);/* LED OFF */
			}
		}
		if (led->mask & 0x0004)
		{
			if (led->onoff)
			{
				GPIO_SetBits(GPIOD, GPIO_Pin_14);	/* LED ON */
			}
			else
			{
				GPIO_ResetBits(GPIOD, GPIO_Pin_14);/* LED OFF */
			}
		}
		if (led->mask & 0x0008)
		{
			if (led->onoff)
			{
				GPIO_SetBits(GPIOD, GPIO_Pin_15);	/* LED ON */
			}
			else
			{
				GPIO_ResetBits(GPIOD, GPIO_Pin_15);/* LED OFF */
			}
		}
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
t_s8 dev_led_0_standby(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
t_s8 dev_led_0_resume(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_Init(GPIOD, &GPIO_InitStructure);
	
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
t_bool dev_led_0_allowstandby(void)
{
	return m_true;
}

/*
** Initialize led device
*/
void dev_led_0_init(void)
{	
	led_0_dev.dev_type		= CORT_DAIT_LED;
	led_0_dev.dev_idx		= 0;
	led_0_dev.open			= dev_led_0_open;;
	led_0_dev.ioctrl		= dev_led_0_ioctrl;
	led_0_dev.standby	 	= dev_led_0_standby;
	led_0_dev.resume	 	= dev_led_0_resume;
	led_0_dev.allow_standby = dev_led_0_allowstandby;
	
	CORT_DAI_RegDev(&led_0_dev);
}

