/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

 #include "config.h"


static t_u8     uart_1_t_buf[64];
static t_u8     uart_1_r_buf[64];
static t_pipe   uart_1_t_pipe;
static t_pipe   uart_1_r_pipe;
static t_device uart_1_dev = {0};
static t_bool   uart_1_init = m_false;
static t_bool   uart_1_standby = m_true;

/*
**  Interrupt Service
*/
static void dev_uart_1_irq(t_u16 idx)
{
	t_u8 chr;
	
	if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET)
	{
		/* Read one byte from the receive data register */
		chr = (t_u8)USART_ReceiveData(USART1);         
		
		/* Write char to queue */
		{
			t_s32 num = 1;
			CORT_Pipe_Write(&uart_1_r_pipe, &chr, &num, 0);
		}
		/* Clear the USARTx Receive interrupt */
		USART_ClearITPendingBit(USART1, USART_IT_RXNE);
	}

	if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET)
	{
		/* Write one byte to the transmit data register */   
		t_s32 num = 1;
		if (CORT_Pipe_Read(&uart_1_t_pipe, &chr, &num, -1) == CORT_OK)
		{
			USART_SendData(USART1, (int)chr);
		}
		else
		{
			/* Disable the USART1 Transmit interrupt */
			USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
		}    
	}
}

/*
** Open Device
*/
static t_s8 dev_uart_1_open(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	if (!uart_1_init)
	{
		uart_1_init = m_true;
		uart_1_standby = m_false;
		
		/* Enable UART clock */
		RCC_APB2PeriphClockCmd(RCC_APB2Periph_USART1, ENABLE);

		/* Connect PXx to USARTx_Tx*/
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource6, GPIO_AF_USART1);

		/* Connect PXx to USARTx_Rx*/
		GPIO_PinAFConfig(GPIOB, GPIO_PinSource7, GPIO_AF_USART1);

		/* Configure USART Tx and Rx as alternate function push-pull */
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
		GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
		GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = GPIO_Pin_7;
		GPIO_Init(GPIOB, &GPIO_InitStructure);

		/* Enable the USARTx Interrupt */
		NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
		NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
		NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
		NVIC_Init(&NVIC_InitStructure);
	  
		/* USARTx configuration */
		USART_InitStructure.USART_BaudRate = 115200;
		USART_InitStructure.USART_WordLength = USART_WordLength_8b;
		USART_InitStructure.USART_StopBits = USART_StopBits_1;
		USART_InitStructure.USART_Parity = USART_Parity_No ;
		USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
		USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
		/* USART configuration */
		USART_Init(USART1, &USART_InitStructure);

		/* Enable USARTx Receive and Transmit interrupts */
		USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
		USART_ITConfig(USART1, USART_IT_TXE, ENABLE);

		/* Enable USART */
		USART_Cmd(USART1, ENABLE);
	}
	return CORT_OK;
}

/*
** Read for uart
*/
static t_s32 dev_uart_1_read(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s8 ret = CORT_Pipe_Read(&uart_1_r_pipe, pBuf, &nLen, tick);

	if (ret == CORT_OK ||
		ret == CORT_ERR_TIMEOUT)
	{
		return nLen;
	}
	return -1;
}

/*
** Write to uart
*/
static t_s32 dev_uart_1_write(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s8 ret = CORT_Pipe_Write(&uart_1_t_pipe, pBuf, &nLen, tick);

	if (ret == CORT_OK ||
		ret == CORT_ERR_TIMEOUT)
	{
		if (nLen)
		{
			if (!USART_GetITStatus(USART1, USART_IT_TXE))
			{
				/* Enable the USARTx Transmit interrupt */
				USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
			}
		}
		return nLen;
	}
	return -1;
}

/*
** IO Control
*/
static t_s8 dev_uart_1_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_uart_set * uart = (t_uart_set*)hParam;
	
	if (uart && (nParam == eIOCtl_UART_SET))
	{
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
static t_s8 dev_uart_1_standby(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	/* Disable USART */
	USART_Cmd(USART1, DISABLE);
	
	/* Configure USART Tx as alternate function  */
	GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART Rx as alternate function  */
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	uart_1_standby = m_true;
	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
static t_s8 dev_uart_1_resume(void)
{	
	GPIO_InitTypeDef GPIO_InitStructure;

	uart_1_standby = m_false;

	/* Connect PXx to USARTx_Tx*/
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource9,  GPIO_AF_USART1);
	/* Connect PXx to USARTx_Rx*/
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource10, GPIO_AF_USART1);
	
	/* Configure USART Tx as alternate function  */
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_UP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART Rx as alternate function  */
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Enable USART */
	USART_Cmd(USART1, ENABLE);
	
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
static t_bool dev_uart_1_allow_standby(void)
{
	return uart_1_standby;
}

/*
** Initialize led device
*/
void dev_uart_1_init(void)
{	
	CORT_Interrupt_Register(53, dev_uart_1_irq);

	CORT_Pipe_Create(&uart_1_t_pipe, uart_1_t_buf, sizeof(uart_1_t_buf));
	CORT_Pipe_Create(&uart_1_r_pipe, uart_1_r_buf, sizeof(uart_1_r_buf));
	
	uart_1_dev.dev_type		= CORT_DAIT_UART;
	uart_1_dev.dev_idx		= 1;
	uart_1_dev.open			= dev_uart_1_open;
	uart_1_dev.read			= dev_uart_1_read;
	uart_1_dev.write		= dev_uart_1_write;
	uart_1_dev.ioctrl	 	= dev_uart_1_ioctrl;
	uart_1_dev.standby	 	= dev_uart_1_standby;
	uart_1_dev.resume	 	= dev_uart_1_resume;
	uart_1_dev.allow_standby= dev_uart_1_allow_standby;
	
	CORT_DAI_RegDev(&uart_1_dev);
}


