/*
 * Copyright (C) Yao Jun 2013
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "config.h"

static t_u16    uart_1_t_front;
static t_u16    uart_1_t_rear;
static t_u8     uart_1_t_buf[64];
static t_u8     uart_1_r_buf[64];
static t_pipe   uart_1_r_pipe;
static t_device uart_1_dev = {0};
static t_bool   uart_1_init = m_false;
static t_bool   uart_1_standby = m_true;

/*
**  Interrupt Service
*/
static void dev_uart_1_irq(t_u16 idx)
{
	t_u8 chr;
	t_s32 num;
	t_u16 sFront;
	
	if (idx == 23)
	{
		chr = UART2_ReceiveData8();
		UART2_ClearITPendingBit(UART2_IT_RXNE);

		/* Write char to queue */
		num = 1;
		CORT_Pipe_Write(&uart_1_r_pipe, &chr, &num, 0);
	}
	if (idx == 22)
	{
		if(uart_1_t_rear != uart_1_t_front)
		{
			sFront = (uart_1_t_front+1) % sizeof(uart_1_t_buf);
			 
			UART2_SendData8(uart_1_t_buf[sFront]);
			
			uart_1_t_front = sFront;
		}
		else
		{
			UART2_ITConfig(UART2_IT_TXE, DISABLE);
		}
	}
}

/*
** Open Device
*/
static t_s8 dev_uart_1_open(void)
{
	if (!uart_1_init)
	{
		uart_1_init = m_true;
		uart_1_standby = m_false;

	    UART2_DeInit();
	    UART2_Init((uint32_t)115200, UART2_WORDLENGTH_8D, UART2_STOPBITS_2, UART2_PARITY_NO,
	                UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);
	    
		UART2_ITConfig(UART2_IT_RXNE, ENABLE);
	}
	return CORT_OK;
}

/*
** Read for uart
*/
static t_s32 dev_uart_1_read(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s8 ret = CORT_Pipe_Read(&uart_1_r_pipe, pBuf, &nLen, tick);

	if (ret == CORT_OK ||
		ret == CORT_ERR_TIMEOUT)
	{
		return nLen;
	}
	return -1;
}

/*
** Write to uart
*/
static t_s32 dev_uart_1_write(t_u8* pBuf, t_s32 nLen, t_s32 tick)
{
	t_s32 i,ret = 0;
	t_u16 sRear;

	for (i = 0; i < nLen; i++)
	{
		sRear = (uart_1_t_rear+1) % sizeof(uart_1_t_buf);

		if( sRear != uart_1_t_front)
		{
			ret++;
			uart_1_t_buf[sRear] = *pBuf++;
			uart_1_t_rear = sRear;
		}
		else
		{
			break;
		}
	}
	if (ret)
	{
		if (!UART2_GetITStatus(UART2_IT_TXE))
		{
			 UART2_ITConfig(UART2_IT_TXE, ENABLE);
		}
	}
	return ret;
}

/*
** IO Control
*/
static t_s8 dev_uart_1_ioctrl(t_u8 nParam, t_void* hParam, t_s32 nTimOutTick)
{
	t_uart_set * uart = (t_uart_set*)hParam;
	
	if (uart && (nParam == eIOCtl_UART_SET))
	{
		return CORT_OK;
	}
	return CORT_ERR_INVALID_PARAM;
}

/*
** Standby Device
*/
static t_s8 dev_uart_1_standby(void)
{
	uart_1_standby = m_true;
	
	return CORT_OK;
}

/*
** Resume Device for Standby
*/
static t_s8 dev_uart_1_resume(void)
{
	uart_1_standby = m_false;
	
	return CORT_OK;
}

/*
** Returns whether to allow the standby state
*/
static t_bool dev_uart_1_allow_standby(void)
{
	return uart_1_standby;
}

/*
** Initialize led device
*/
void dev_uart_1_init(void)
{	
	CORT_Interrupt_Register(22, dev_uart_1_irq);
	CORT_Interrupt_Register(23, dev_uart_1_irq);

	CORT_Pipe_Create(&uart_1_r_pipe, uart_1_r_buf, sizeof(uart_1_r_buf));

	uart_1_t_front = 0;
	uart_1_t_rear  = 0;

	uart_1_dev.dev_type		= CORT_DAIT_UART;
	uart_1_dev.dev_idx		= 1;
	uart_1_dev.open			= dev_uart_1_open;
	uart_1_dev.read			= dev_uart_1_read;
	uart_1_dev.write		= dev_uart_1_write;
	uart_1_dev.ioctrl	 	= dev_uart_1_ioctrl;
	uart_1_dev.standby	 	= dev_uart_1_standby;
	uart_1_dev.resume	 	= dev_uart_1_resume;
	uart_1_dev.allow_standby= dev_uart_1_allow_standby;
	
	CORT_DAI_RegDev(&uart_1_dev);
}

