/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "cortos.h"
#include "cortos_common.h"
#include "config.h"

#define MT(x0,x1)	((x0)|(x1<<4))

const t_u8 CORT_PrioMapTbl[128] = {
	/*             0 1      2 3      4 5      6 7      8 9      A B      C D      E F*/
	/* 0x00 */  MT(0,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x10 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x20 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x30 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x40 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x50 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x60 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x70 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x80 */  MT(7,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0x90 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xA0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xB0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xC0 */  MT(6,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xD0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xE0 */  MT(5,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0),
    /* 0xF0 */  MT(4,0), MT(1,0), MT(2,0), MT(1,0), MT(3,0), MT(1,0), MT(2,0), MT(1,0)
};

       t_u8  CORT_IrqIdx = 0;
/*
**
*/
void   CORT_EnterCritical(void)
{
	sim();
}

/*
**
*/
void   CORT_ExitCritical(void)
{
	rim();
}

/*
**
*/
t_u8  CORT_CountLeadZeros(t_prio val)
{
	t_u8 prio = CORT_PrioMapTbl[val>>1];
	if (val & 0x01)
	{
		prio >>= 4;
	}
	return prio & 0x0F;
}
/*
**
*/
t_stk *CORT_InitStk(void (*task)(void *p_arg), void *parg, t_stk *pstk, t_u32 size)
{
  	t_u16 temp16;
	t_u32 temp32;
	t_u8 *stk = (t_u8*)pstk + (size-2);
	
	temp32  = (t_u32)task;
	*stk--  = (t_u8 )(temp32 & 0xff);      /* PCL */
	temp32  = temp32 >> 8;
	*stk--  = (t_u8 )(temp32 & 0xff);      /* PCH */
	temp32  = temp32 >> 8;
	*stk--  = (t_u8 )(temp32 & 0xff);      /* PCE */
	
	// NOR REGS
	*stk--  = (t_u8 )(0x01);               /* YL */
	*stk--  = (t_u8 )(0x02);               /* YL */
	temp16  = 0x0000;//(t_u16)parg;
	*stk--  = (t_u8 )(temp16 & 0xff);      /* XL */
	temp16  = temp16 >> 8;
	*stk--  = (t_u8 )(temp16 & 0xff);      /* XH */
	*stk--  = (t_u8 )(0x03);               /* A */
	*stk--  = (t_u8 )(0x24);               /* CC */
	
	// EX REGS
	*stk--  = (t_u8 )(0x10);               /* ?B0 */
	*stk--  = (t_u8 )(0x11);               /* ?B1 */
	*stk--  = (t_u8 )(0x12);               /* ?B2 */
	*stk--  = (t_u8 )(0x13);               /* ?B3 */
	*stk--  = (t_u8 )(0x14);               /* ?B4 */
	*stk--  = (t_u8 )(0x15);               /* ?B5 */
	*stk--  = (t_u8 )(0x16);               /* ?B6 */
	*stk--  = (t_u8 )(0x17);               /* ?B7 */
	*stk--  = (t_u8 )(0x18);               /* ?B8 */
	*stk--  = (t_u8 )(0x19);               /* ?B9 */
	*stk--  = (t_u8 )(0x1A);               /* ?B10 */
	*stk--  = (t_u8 )(0x1B);               /* ?B11 */
	*stk--  = (t_u8 )(0x1C);               /* ?B12 */
	*stk--  = (t_u8 )(0x1D);               /* ?B13 */
	*stk--  = (t_u8 )(0x1E);               /* ?B14 */
	*stk--  = (t_u8 )(0x1F);               /* ?B15 */

	/* 25 byte */
	
    return (t_stk*)(stk);
}


/*
**
*/
void CORT_Hook_Tick(void)
{
	/* Cleat Interrupt Pending bit */
	  TIM4_ClearITPendingBit(TIM4_IT_UPDATE);
}
