/*
 * Copyright (C) Yao Jun 2011
 * <nter@tom.com> 
 * <http://CortOS.sourceforge.net/>
 *
 * SortOS is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SortOS is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contents:
 *   CORT_IntEntry()       - 
 */

#include "cortos.h"
#include "cortos_common.h"

typedef void (*interrupt_handler_t)(t_u16);


static t_u16 erridx = 0xffff;
void NonHandledInterrupt(t_u16 idx)
{
	erridx = idx;
	erridx = erridx;
	while(1);
}

static interrupt_handler_t vec_tab[] = {
	{(interrupt_handler_t)NonHandledInterrupt},		/* 0 RESET */
	{(interrupt_handler_t)NonHandledInterrupt},		/* 1 TRAP - Software interrupt */
	{(interrupt_handler_t)NonHandledInterrupt},		/* 2 irq0 - External Top Level interrupt (TLI) */
	{(interrupt_handler_t)NonHandledInterrupt},		/* 3 irq1 - Auto Wake Up from Halt interrupt */
	{(interrupt_handler_t)NonHandledInterrupt},		/* 4 irq2 - Clock Controller interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/* 5 irq3 - External interrupt 0 (GPIOA) */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/* 6 irq4 - External interrupt 1 (GPIOB) */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/* 7 irq5 - External interrupt 2 (GPIOC) */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/* 8 irq6 - External interrupt 3 (GPIOD) */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/* 9 irq7 - External interrupt 4 (GPIOE) */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*10 irq8 - Reserved */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*11 irq9 - Reserved */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*12 irq10 - SPI End of transfer interrupt */
	{(interrupt_handler_t)NonHandledInterrupt},		/*13 irq11 - TIM1 Update/Overflow/Trigger/Break interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*14 irq12 - TIM1 Capture/Compare interrupt */  
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*15 irq13 - TIM2 Update/Overflow/Break interrupt  */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*16 irq14 - TIM2 Capture/Compare interrupt */	
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*17 irq15 - TIM3 Update/Overflow/Break interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*18 irq16 - TIM3 Capture/Compare interrupt */	
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*19 irq17 - Reserved */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*20 irq18 - Reserved */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*21 irq19 - I2C interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*22 irq20 - UART2 Tx interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*23 irq21 - UART2 Rx interrupt */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*24 irq22 - ADC1 end of conversion/Analog watchdog interrupts */
	{(interrupt_handler_t)NonHandledInterrupt}, 	/*25 irq23 - TIM4 Update/Overflow interrupt */
	{(interrupt_handler_t)NonHandledInterrupt},  	/*26 irq24 - FLASH interrupt */
};

/*
**
*/
void CORT_IntRoutine(t_u8 id)
{
	if (id > 1 && id < CORT_ARRAYSIZE(vec_tab)){
		vec_tab[id](id);
	}
}

/*
**
*/
t_s8  CORT_Interrupt_Register(t_u16 idx,void (*task_entry)(t_u16))
{
	CORT_EnterCritical();
	if (idx > 1 && idx < CORT_ARRAYSIZE(vec_tab))
	{
		vec_tab[idx] = task_entry;
		CORT_ExitCritical();
		return CORT_OK;
	}
	CORT_ExitCritical();
	return CORT_ERROR;
}
